import grpc
from github.com.metaprov.modelaapi.pkg.apis.team.v1alpha1.generated_pb2 import Meeting as MDMeeting
from github.com.metaprov.modelaapi.services.meeting.v1.meeting_pb2_grpc import MeetingServiceStub
from github.com.metaprov.modelaapi.services.meeting.v1.meeting_pb2 import CreateMeetingRequest, \
    UpdateMeetingRequest, \
    DeleteMeetingRequest, GetMeetingRequest, ListMeetingsRequest

from modela.Resource import Resource
from modela.ModelaException import ModelaException
from typing import List, Union


class Meeting(Resource):
    def __init__(self, item: MDMeeting = MDMeeting(), client=None, namespace="", name=""):
        super().__init__(item, client, namespace=namespace, name=name)


class MeetingClient:
    def __init__(self, stub, modela):
        self.modela = modela
        self.__stub: MeetingServiceStub = stub

    def create(self, meeting: Meeting) -> bool:
        request = CreateMeetingRequest()
        request.meeting.CopyFrom(meeting.raw_message)
        try:
            response = self.__stub.CreateMeeting(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def update(self, meeting: Meeting) -> bool:
        request = UpdateMeetingRequest()
        request.meeting.CopyFrom(meeting.raw_message)
        try:
            self.__stub.UpdateMeeting(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def get(self, namespace: str, name: str) -> Union[Meeting, bool]:
        request = GetMeetingRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.GetMeeting(request)
            return Meeting(response.meeting, self)
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def delete(self, namespace: str, name: str) -> bool:
        request = DeleteMeetingRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.DeleteMeeting(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def list(self, namespace: str) -> Union[List[Meeting], bool]:
        request = ListMeetingsRequest()
        request.namespace = namespace
        try:
            response = self.__stub.ListMeetings(request)
            return [Meeting(item, self) for item in response.meetings.items]
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False





import grpc
from github.com.metaprov.modelaapi.pkg.apis.data.v1alpha1.generated_pb2 import WebRequest as MDWebRequest
from github.com.metaprov.modelaapi.services.webrequest.v1.webrequest_pb2_grpc import WebRequestServiceStub
from github.com.metaprov.modelaapi.services.webrequest.v1.webrequest_pb2 import CreateWebRequestRequest, \
    UpdateWebRequestRequest, \
    DeleteWebRequestRequest, GetWebRequestRequest, ListWebRequestsRequest

from modela.Resource import Resource
from modela.ModelaException import ModelaException
from typing import List, Union


class WebRequest(Resource):
    def __init__(self, item: MDWebRequest = MDWebRequest(), client=None, namespace="", name="", version=Resource.DefaultVersion):
        super().__init__(item, client, namespace=namespace, name=name, version=version)


class WebRequestClient:
    def __init__(self, stub, modela):
        self.modela = modela
        self.__stub: WebRequestServiceStub = stub

    def create(self, webrequest: WebRequest) -> bool:
        request = CreateWebRequestRequest()
        request.webrequest.CopyFrom(webrequest.raw_message)
        try:
            response = self.__stub.CreateWebRequest(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def update(self, webrequest: WebRequest) -> bool:
        request = UpdateWebRequestRequest()
        request.webrequest.CopyFrom(webrequest.raw_message)
        try:
            self.__stub.UpdateWebRequest(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def get(self, namespace: str, name: str) -> Union[WebRequest, bool]:
        request = GetWebRequestRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.GetWebRequest(request)
            return WebRequest(response.webrequest, self)
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def delete(self, namespace: str, name: str) -> bool:
        request = DeleteWebRequestRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.DeleteWebRequest(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def list(self, namespace: str) -> Union[List[WebRequest], bool]:
        request = ListWebRequestsRequest()
        request.namespace = namespace
        try:
            response = self.__stub.ListWebRequests(request)
            return [WebRequest(item, self) for item in response.webrequests.items]
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False





from dataclasses import field
from typing import List
import github.com.metaprov.modelaapi.pkg.apis.catalog.v1alpha1.generated_pb2 as catalog_pb
import github.com.metaprov.modelaapi.pkg.apis.inference.v1alpha1.generated_pb2 as inference_pb
from github.com.metaprov.modelaapi.services.grpcinferenceservice.v1.grpcinferenceservice_pb2 import *
from modela.Configuration import datamodel
from modela.common import Configuration, ConditionStatus, Time, ObjectReference, StatusError, TriggerScheduleEventType,TestSuite
from modela.inference.common import PredictorConditionType, AccessType, PredictorType, ModelDeploymentPhase, ModelRole
from modela.infra.models import Workload
from modela.data.models import DataLocation
from modela.training.common import TaskType
from modela.training.models import ModelTest, ModelTestResult
from modela.common import Metric


@datamodel(proto=catalog_pb.RunSchedule)
class RunSchedule(Configuration):
    """ RunSchedule specifies the schedule for a Job to be executed """
    Enabled: bool = False
    """ Indicates if the schedule is enabled and the Jobs associated it will be created at the specified time """
    StartTime: Time = None
    """ The time of the day when the schedule will be executed """
    EndTime: Time = None
    """ The time of the day when the schedule is expected to conclude """
    Cron: str = ''
    """ The cron string of the schedule. See https://docs.oracle.com/cd/E12058_01/doc/doc.1014/e12030/cron_expressions.htm for more information """
    Type: TriggerScheduleEventType = None
    """ The type of schedule, which can be a frequency interval or a cron expression """


@datamodel(proto=inference_pb.ModelDeploymentStatus)
class ModelDeploymentStatus(Configuration):
    ImageName: str = ''
    ModelName: str = ''
    """ The name of the Model resource associated with the deployment """
    ModelVersion: str = ''
    """ The version of the Model resource """
    HpaRef: ObjectReference = None
    """ the name of the horizonal pod autoscaler, if autoscaling is true """
    DeploymentRef: ObjectReference = None
    """ The name of the Kubernetes Deployment that manages the pods of the Model """
    ServiceRef: ObjectReference = None
    """ The name of the Kubernetes Service which exposes the Model externally """
    P50: float = 0
    """ 50% latency (median) for predictions served by the model """
    P95: float = 0
    """ 95% latency for predictions served by the model """
    P99: float = 0
    """ 99% latency for predictions served by the model """
    LastPredictionTime: Time = None
    """ The last time a prediction was served by the Predictorlet """
    DailyPredictionAvg: int = 0
    LastFailure: str = ''
    """ LastFailure is the last failure that occured with the model """
    Phase: ModelDeploymentPhase = None
    """ Phase is the current phase of the model deployment """
    DeployedAt: Time = None
    """ DeployedAt is the last time that the model was deployed """
    ReleasedAt: Time = None
    """ ReleasedAt is the time that the model was released """
    DataDrift: bool = False
    """ Indicates if a data drift has been detected based on incoming prediction data """
    ConceptDrift: bool = False
    """ Indicates if a concept drift has been detected based on incoming prediction data """
    LastDailyPredictions: List[int] = field(default_factory=lambda : [])
    """ The predictions from the last 7 days """


@datamodel(proto=inference_pb.PredictorletStatus)
class PredictorletStatus(Configuration):
    """ PredictorletStatus describes the current state of a prediction proxy service associated with a Predictor """
    ImageName: str = ''
    """ The image name that the Predictorlet is currently running """
    DeploymentRef: ObjectReference = None
    """ The name of the Kubernetes Deployment that manages the pods of the Predictorlet """
    ServiceRef: ObjectReference = None
    """ The name of the Kubernetes Service which exposes the Predictorlet externally """
    P50: float = 0
    """ 50% latency (median) for predictions served by the Predictorlet """
    Current95: float = 0
    """ 95% latency for predictions served by the Predictorlet """
    Current99: float = 0
    """ 99% latency for predictions served by the Predictorlet """
    LastPredictionTime: Time = None
    """ The last time a prediction was served by the Predictorlet """
    DailyPredictionAvg: int = 0
    TotalPredictions: int = 0
    """ The total number of predictions served by the Predictorlet """
    LastFailure: str = ''
    """ LastFailure contains the last failure that occured with a model served by the Predictorlet """
    LastDailyPredictions: List[int] = field(default_factory=lambda : [])
    """ The predictions from the last 7 days """


@datamodel(proto=inference_pb.ModelRecord)
class ModelRecord(Configuration):
    """ ModelRecord hold the state of a model that was in production """
    ModelName: str = ''
    """ Model Name is the name of the model """
    ModelVersion: str = ''
    """ Model version is the version of the model """
    LiveAt: Time = None
    """ Live at is the time that the model were placed in production """
    RetiredAt: Time = None
    """ Retried at is the time that the model was retired from production. """
    AvgDailyPrediction: int = 0
    """ Retried at is the time that the model was retired from production. """
    AvgLatency: float = 0
    """ Retried at is the time that the model was retired from production. """


@datamodel(proto=catalog_pb.ModelDeploymentSpec)
class ModelDeploymentSpec(Configuration):
    """
    ModelDeploymentSpec describes how a single model should be deployed with a Predictor, and
    how prediction traffic will be routed to the model
    """
    ModelRef: ObjectReference = None
    """
    The reference to a Model resource which has been packaged and exists in the same Data Product
    as the Predictor which specifies the ModelDeploymentSpec
    """
    ModelVersion: str = ''
    """ The version of the model, derived from the Study which created it """
    Traffic: int = 0
    """ The minimum percentage (0 through 100) of traffic that will be served by the model """
    Role: ModelRole = ModelRole.Champion
    """ Role denotes the role of this model """
    Released: bool = False
    """ A released model is a model that should serve production traffic """
    Deployed: bool = False
    """ A deployed model is a model whose containers are up, but does not serve production traffic """
    MountTar: bool = False
    """ MountTar means that we would mount the model tar file. Else we would use baked image. """
    TrafficSelector: str = ''
    """ TrafficSelector is a filter on the traffic to this model """
    CanaryMetrics: List[Metric] = field(default_factory=lambda : [])
    """ If the deployment is canary, the metric define how to evaluate the canary """
    ApprovedBy: str = ''
    """ The account name of the approver """
    ApprovedAt: Time = None
    """ The time of approval """


@datamodel(proto=inference_pb.PredictorCondition)
class PredictorCondition(Configuration):
    """ PredictorCondition describes the state of a Predictor at a certain point """
    Type: PredictorConditionType = None
    """ Type of Predictor condition """
    Status: ConditionStatus = None
    """ Status of the condition, one of True, False, Unknown """
    LastTransitionTime: Time = None
    """ Last time the condition transitioned from one status to another """
    Reason: str = ''
    """ The reason for the condition's last transition """
    Message: str = ''
    """ A human-readable message indicating details about the transition """


@datamodel(proto=inference_pb.ProgressiveSpec)
class ProgressiveSpec(Configuration):
    """ ProgressiveSpec defines the specification to progressively deploy a model to production """
    Warmup: int = 0
    """ The time, in seconds, for the warm-up period """
    TrafficIncrement: int = 0
    """ The percentage of traffic to increment """
    CanaryMetrics: List[Metric] = field(default_factory=lambda : [])
    """ What metric to use when comparing the candidate model to the current model """


@datamodel(proto=inference_pb.PredictionCacheSpec)
class PredictionCacheSpec(Configuration):
    """
    ///////////////////////////////////////////////////
    Prediction Cache Spec
    ///////////////////////////////////////////////////
    PredictionCacheSpec specifies the connection information of a key-value cache to store predictions
    """
    Enabled: bool = False
    """ Enabled indicates if predictions will be cached """
    InMemory: bool = False
    """ InMemory indicates if predictions will be cached in the available memory of the Pod serving the model """
    Redis: bool = False
    """ Redis indicates if predictions will be cached in an external Redis deployment """
    Hostname: ObjectReference = None
    """ The reference to a Connection resource to an external Redis deployment """


@datamodel(proto=inference_pb.AutoScalingSpec)
class AutoScalingSpec(Configuration):
    """
    ///////////////////////////////////////////////////
    Auto Scaling Spec
    ///////////////////////////////////////////////////
    AutoScaling defines the configuration for the automatic scaling of a service
    """
    Enabled: bool = False
    """ Indicates if automatic scaling is enabled """
    MinReplicas: int = 1
    """ The minimum number of replicas running the service """
    MaxReplicas: int = 1
    """ The maximum number of replicas running the service """
    CpuAvgUtilization: int = 80
    """
    The target average CPU utilization across all replicas. The HorizontalPodAutoscaler associated with the
    service will change the number of replicas to maintain this metric
    """
    MemAvgUtilization: int = 80
    """
    The target average memory utilization across all replicas. The HorizontalPodAutoscaler associated with the
    service will change the number of replicas to maintain this metric
    """


@datamodel(proto=inference_pb.PredictorStatus)
class PredictorStatus(Configuration):
    """ PredictorStatus contain the current state of the Predictor resource """
    ObservedGeneration: int = 0
    """ ObservedGeneration is the last generation that was acted on """
    History: List[ModelRecord] = field(default_factory=lambda : [])
    """ The collection of historical records of models deployed to the Predictor, used internally to roll-back models """
    Models: List[ModelDeploymentStatus] = field(default_factory=lambda : [])
    """ The collection of statuses for each model deployed with the Predictor """
    Predictorlet: PredictorletStatus = None
    """
    The status of the Predictorlet associated with the Predictor. The Predictorlet is a service which handles prediction traffic
    and routes predictions to individual models based on the specification of the Predictor
    """
    LastUpdated: Time = None
    """ The last time the object was updated """
    EndPoint: str = ''
    """ The end-point URL of the Predictor """
    FailureReason: StatusError = None
    """ In the case of failure, the Predictor resource controller will set this field with a failure reason """
    FailureMessage: str = ''
    """ In the case of failure, the Predictor resource controller will set this field with a failure reason """
    Conditions: List[PredictorCondition] = field(default_factory=lambda : [])


@datamodel(proto=inference_pb.PredictorMonitorSpec)
class PredictorMonitorSpec(Configuration):
    """
    MonitorSpec defines the specification to monitor a model in production
    """
    Enabled: bool = False
    """
    Indicates if model monitoring is enabled for the model
    """
    Tests:TestSuite = TestSuite()
    """
    tests contains the collection of model tests that will be 
    performed based on incoming prediction traffic
    """
    Schedule: RunSchedule = None
    """ The schedule on which model monitoring computations will be performed """
    NotifierRef: ObjectReference = None
    """ NotifierRef references a Notifier resource that will be triggered in the case that a concept or data drift is detected """
    OutlierDetectionModelRef: ObjectReference = None
    """ Reference to a model that will be used for outlier detection. If empty, an outlier detection model. """


@datamodel(proto=inference_pb.PredictionLoggingSpec)
class PredictionLoggingSpec(Configuration):
    """ PredictionLoggingSpec specifies the configuration to log incoming and outgoing prediction requests """
    Enabled: bool = False
    """ Indicates if prediction logging is enabled """
    SamplePercent: int = 0
    """ The number percentage (0 through 100) of prediction requests to log """
    LogRequests: bool = True
    """ Indicates if incoming requests will be logged """
    LogResponses: bool = True
    """ Indicates if outgoing predictions will be logged """


@datamodel(proto=inference_pb.ModelServingSpec)
class ModelServingSpec(Configuration):
    """ ModelServingSpec specifies the configuration for models to be served by a Predictor """
    Type: PredictorType = PredictorType.Online
    """ The type of predictor (online, batch, or streaming). Online is the only supported type as of the current release """
    Serverless: bool = False
    """
    If Serverless is true, the Kubernetes Deployment which serves the model will not be created 
    until it starts to receive prediction traffic, and will be destroyed once the model becomes dormant
    """


@datamodel(proto=inference_pb.ForwardCurtainSpec)
class ForwardCurtainSpec(Configuration):
    """ AccessSpec specifies the configuration to expose a Predictor service externally """
    Enabled: bool = False
    """
    Is Forward curtain spec enabled
    """
    CurtainRef:ObjectReference = None
    """
    The forward curtain receives prediction requests before the prediction (currently unimplemented)
    """
    Percent: int = 0
    """
    Percent of request that are sent to the foreward curtain.
    """

@datamodel(proto=inference_pb.ForwardCurtainSpec)
class BackwardCurtainSpec(Configuration):
    """ AccessSpec specifies the configuration to expose a Predictor service externally """
    Enabled: bool = False
    """
    Is Forward curtain spec enabled
    """
    CurtainRef:ObjectReference = None
    """
    The forward curtain receives prediction requests before the prediction (currently unimplemented)
    """
    ConfidenceLow: float = 0.4
    """
    For backward curtain is the confidence low
    """
    ConfidenceHigh: float = 0.6
    """
    For backward curtain is the confidence high
    """

@datamodel(proto=inference_pb.OnlineFeatureStoreSpec)
class OnlineFeatureStoreSpec(Configuration):
    """ AccessSpec specifies the configuration to expose a Predictor service externally """
    Enabled: bool = False
    """
    Is online feature store enabled
    """
    HostName:str = ""
    """
    The forward curtain receives prediction requests before the prediction (currently unimplemented)
    """

@datamodel(proto=inference_pb.FastSlowModelSpec)
class FastSlowModelSpec(Configuration):
    """ AccessSpec specifies the configuration to expose a Predictor service externally """
    Enabled: bool = False
    """
    Is fast slow enabled
    """
    FastModelRef: ObjectReference = None
    """
    Reference to the fast model
    """
    SlowModelRef:ObjectReference = None
    """
    Reference to the slow model
    """
    ProbaLowPct:int = 40
    """
    The low range of confidence.
    """
    ProbaHighPct:int = 60
    """
    The high range of confidence , Must be higher than probalow
    """


AccessType_ = AccessType


@datamodel(proto=inference_pb.AccessSpec)
class AccessSpec(Configuration):
    """ AccessSpec specifies the configuration to expose a Predictor service externally """
    Port: int = 0
    """
    The port number that will be exposed on the Predictor's Pods to serve prediction traffic through the GRPCInferenceService API.
    The Kubernetes Service created by the Predictor will expose the port and forward GRPC traffic to the backend pods
    """
    NodePort: int = 0
    """
    The port number that will be exposed on the external address of every node on the cluster, in the case of the
    Predictor's access type being NodePort. Traffic from the port will be forwarded to the Predictor's backend service
    """
    Path: str = 0
    """
    The auto-generated DNS path where the Predictor service can be accessed. If the access type is ClusterIP, it will
    be a cluster-internal DNS name (i.e. predictor.default-serving-site.svc.cluster.local). In the case of the Ingress
    access type, it will be determined by the FQDN of the host ServingSite (i.e. predictor.default-serving-site.your-domain.ai).
    """
    AccessType: AccessType_ = AccessType_.ClusterIP
    """
    The Kubernetes-native access method which specifies how the Kubernetes Service created by the Predictor will be exposed.
    See https://modela.ai/docs/docs/serving/production/#access-method for a detailed description of each access type
    (defaults to cluster-ip)
    """
    Rest: bool = False
    """
    Indicates if the prediction service should expose an additional port to serve the GRPCInferenceService API through REST.
    The port one digit above the number specified by the Port field will be exposed to accept HTTP/1.1 traffic
    """
    ApikeySecretRef: ObjectReference = None
    """ ApiKeySecretRef references a Kubernetes Secret containing an API key that must be passed in prediction requests to the Predictor """


@datamodel(proto=inference_pb.PredictorSpec)
class PredictorSpec(Configuration):
    """ PredictorSpec defines the desired state of a Predictor """
    VersionName: str = ''
    """
    The name of the DataProductVersion which describes the version of the resource
    that exists in the same DataProduct namespace as the resource
    """
    Description: str = ''
    """ The user-provided description of the Predictor """
    ProductRef: ObjectReference = None
    """ The reference to the DataProduct that the resource exists under """
    ServingsiteRef: ObjectReference = ObjectReference('default-tenant', 'default-lab')
    """ The reference to the ServingSite resource that hosts the Predictor """
    Models: List[ModelDeploymentSpec] = field(default_factory=lambda : [])
    """
    The collection of model deployment specifications that define which Model resources will be deployed to the
    Predictor service and how they will be deployed. Each model should be trained with the same type of
    dataset and possess a unique version
    """
    Progressive: ProgressiveSpec = None
    """
    The specification to progressively deploy models. ModelDeploymentSpec specifications within Models that have the
    `Canary` field enabled will be progressively deployed according to the specification when they are applied to the Predictor
    """
    ArtifactsFolder: DataLocation = DataLocation()
    """ The data location where artifacts generated by the Predictor will be stored """
    Access: AccessSpec = AccessSpec()
    """ Access specifies the configuration for the Predictor service to be exposed externally """
    Serving: ModelServingSpec = ModelServingSpec()
    """ Serving specifies the configuration for individual models to handle prediction traffic """
    PredictionLogging: PredictionLoggingSpec = PredictionLoggingSpec()
    """ Monitor spec specify the monitor for this predictor. """
    Replicas: int = 1
    """
    The number of replicas for the Kubernetes Deployment associated with the Predictor, which will instantiate multiple
    copies of the service in the case that automatic scaling is disabled
    """
    AutoScaling: AutoScalingSpec = None
    """ AutoScaling specifies the auto-scaling policy """
    Owner: str = 'no-one'
    """ The name of the Account which created the object, which exists in the same tenant as the object """
    Resources: Workload = Workload('general-nano')
    """ Resources specifies the resource requirements allocated to the prediction service """
    Cache: PredictionCacheSpec = PredictionCacheSpec()
    """ Cache specifies the configuration of the prediction cache """
    Store: OnlineFeatureStoreSpec = OnlineFeatureStoreSpec()

    ForwardCurtain: ForwardCurtainSpec = ForwardCurtainSpec()

    BackwardCurtain: BackwardCurtainSpec = BackwardCurtainSpec()

    Task: TaskType = None
    """ The task type of the Predictor, which should match the task type of the models being served """
    PredictionThreshold: float = 0
    """ The prediction threshold """
    Monitor: PredictorMonitorSpec = None
    """ Monitor spec specify the monitor for this predictor. """
    FastSlow: FastSlowModelSpec = FastSlowModelSpec()


@datamodel(proto=ProbabilityValue)
class ProbabilityValue(Configuration):
    Label: str = ''
    Probability: float = 0


@datamodel(proto=ShapValue)
class ShapValue(Configuration):
    Feature: str = ''
    Value: float = 0


@datamodel(proto=PredictResultLineItem)
class PredictionResult(Configuration):
    Success: bool = False
    Score: float = 0
    Label: str = ''
    Probabilities: List[ProbabilityValue] = field(default_factory=lambda : [])
    ShapValues: List[ShapValue] = field(default_factory=lambda : [])
    MissingColumns: List[str] = field(default_factory=lambda : [])
    OutOfBound: List[str] = field(default_factory=lambda : [])
    BaseShapValue: float = 0




@datamodel(proto=inference_pb.BackwardCurtainSpec)
class BackwardCurtainSpec(Configuration):
    """ AccessSpec specifies the configuration to expose a Predictor service externally """
    Port: int = 0
    """
    The port number that will be exposed on the Predictor's Pods to serve prediction traffic through the GRPCInferenceService API.
    The Kubernetes Service created by the Predictor will expose the port and forward GRPC traffic to the backend pods
    """
    NodePort: int = 0
    """
    The port number that will be exposed on the external address of every node on the cluster, in the case of the
    Predictor's access type being NodePort. Traffic from the port will be forwarded to the Predictor's backend service
    """
    Path: str = 0
    """
    The auto-generated DNS path where the Predictor service can be accessed. If the access type is ClusterIP, it will
    be a cluster-internal DNS name (i.e. predictor.default-serving-site.svc.cluster.local). In the case of the Ingress
    access type, it will be determined by the FQDN of the host ServingSite (i.e. predictor.default-serving-site.your-domain.ai).
    """
    AccessType: AccessType_ = AccessType_.ClusterIP
    """
    The Kubernetes-native access method which specifies how the Kubernetes Service created by the Predictor will be exposed.
    See https://modela.ai/docs/docs/serving/production/#access-method for a detailed description of each access type
    (defaults to cluster-ip)
    """
    Rest: bool = False
    """
    Indicates if the prediction service should expose an additional port to serve the GRPCInferenceService API through REST.
    The port one digit above the number specified by the Port field will be exposed to accept HTTP/1.1 traffic
    """
    ApikeySecretRef: ObjectReference = None
    """ ApiKeySecretRef references a Kubernetes Secret containing an API key that must be passed in prediction requests to the Predictor """



import grpc
from github.com.metaprov.modelaapi.pkg.apis.infra.v1alpha1.generated_pb2 import Notifier as MDNotifier
from github.com.metaprov.modelaapi.services.notifier.v1.notifier_pb2_grpc import NotifierServiceStub
from github.com.metaprov.modelaapi.services.notifier.v1.notifier_pb2 import CreateNotifierRequest, \
    UpdateNotifierRequest, \
    DeleteNotifierRequest, GetNotifierRequest, ListNotifiersRequest

from modela.Resource import Resource
from modela.ModelaException import ModelaException
from typing import List, Union


class Notifier(Resource):
    def __init__(self, item: MDNotifier = MDNotifier(), client=None, namespace="", name=""):
        super().__init__(item, client, namespace=namespace, name=name)


class NotifierClient:
    def __init__(self, stub, modela):
        self.modela = modela
        self.__stub: NotifierServiceStub = stub

    def create(self, notifier: Notifier) -> bool:
        request = CreateNotifierRequest()
        request.notifier.CopyFrom(notifier.raw_message)
        try:
            response = self.__stub.CreateNotifier(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def update(self, notifier: Notifier) -> bool:
        request = UpdateNotifierRequest()
        request.notifier.CopyFrom(notifier.raw_message)
        try:
            self.__stub.UpdateNotifier(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def get(self, namespace: str, name: str) -> Union[Notifier, bool]:
        request = GetNotifierRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.GetNotifier(request)
            return Notifier(response.notifier, self)
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def delete(self, namespace: str, name: str) -> bool:
        request = DeleteNotifierRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.DeleteNotifier(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def list(self, namespace: str) -> Union[List[Notifier], bool]:
        request = ListNotifiersRequest()
        request.namespace = namespace
        try:
            response = self.__stub.ListNotifiers(request)
            return [Notifier(item, self) for item in response.notifiers.items]
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False





import grpc
from github.com.metaprov.modelaapi.pkg.apis.infra.v1alpha1.generated_pb2 import UserRoleClass as MDUserRoleClass
from github.com.metaprov.modelaapi.services.userroleclass.v1.userroleclass_pb2_grpc import UserRoleClassServiceStub
from github.com.metaprov.modelaapi.services.userroleclass.v1.userroleclass_pb2 import CreateUserRoleClassRequest, \
    UpdateUserRoleClassRequest, \
    DeleteUserRoleClassRequest, GetUserRoleClassRequest, ListUserRoleClassesRequest

from modela.Resource import Resource
from modela.ModelaException import ModelaException
from modela.infra.models import UserRoleClassSpec, Rule
from modela.infra.common import ResourceKind, Verb
from typing import List, Union


class UserRoleClass(Resource):
    """ User Role Classes hold rules about what actions an Account can perform on different resources. """

    def __init__(self, item: MDUserRoleClass = MDUserRoleClass(), client=None, namespace="", name="",
                 rules: List[Rule] = None):
        """
        :param client: The User Role Class client repository, which can be obtained through an instance of Modela.
        :param namespace: The target namespace of the resource.
        :param name: The name of the resource.
        :param rules: The list of rules which any Account associated with the User Role Class resource may perform.
        """
        super().__init__(item, client, namespace=namespace, name=name)

        if rules is not None:
            self.spec.Rules = rules

    @property
    def spec(self) -> UserRoleClassSpec:
        return UserRoleClassSpec().copy_from(self._object.spec).set_parent(self._object.spec)

    def default(self):
        UserRoleClassSpec().apply_config(self._object.spec)

    @property
    def rules(self):
        """ Returns the list of rules associated with the User Role Class """
        return self.spec.Rules

    def rule(self, resource: ResourceKind) -> Rule:
        """ Returns the rule for an individual resource. Returns an empty rule if the resource does not have one. """
        rule = [rule for rule in self.rules if rule.Resource == resource]
        if len(rule) > 0:
            return rule[0]
        else:
            return Rule(Resource=resource)


    def add_rule(self, rule: Rule):
        """ Adds a rule for a single resource, or merges it if a rule for the resource already exists. """
        existing_rule = [irule for irule in self.rules if irule.Resource == rule.Resource]
        if len(existing_rule) > 0:
            rule.Verbs = list(set(existing_rule[0].Verbs + rule.Verbs))
        else:
            self.spec.Rules.append(rule)

        self.update()

    def allow_verb(self, resource: ResourceKind, verb: Verb):
        """ Sets the rule for the resource to permit the specified verb """
        self.add_rule(Rule(Resource=resource, Verbs=[verb]))

    def deny_verb(self, resource: ResourceKind, verb: Verb):
        """ Sets the rule for the resource to deny the specified verb """
        rule = [rule for rule in self.rules if rule.Resource == resource]
        if len(rule) > 0:
            rule = rule[0]
            if verb.All in rule.Verbs:
                rule.Verbs = [verb.List, verb.Watch, verb.Create, verb.Update, verb.Patch, verb.Delete, verb.Get]

            try:
                rule.Verbs.remove(verb)
            except ValueError:
                pass

            self.update()

    def allow_all(self, resource: ResourceKind):
        """ Permits all operations for the specified resource """
        self.allow_verb(resource, Verb.All)

    def deny_all(self, resource: ResourceKind):
        """ Denies all operations for the specified resource """
        rule = [rule for rule in self.rules if rule.Resource == resource]
        if len(rule) > 0:
            self.rules.remove(rule[0])
            self.update()


class UserRoleClassClient:
    def __init__(self, stub, modela):
        self.modela = modela
        self.__stub: UserRoleClassServiceStub = stub

    def create(self, userroleclass: UserRoleClass) -> bool:
        request = CreateUserRoleClassRequest()
        request.UserRoleClass.CopyFrom(userroleclass.raw_message)
        try:
            response = self.__stub.CreateUserRoleClass(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def update(self, UserRoleClass: UserRoleClass) -> bool:
        request = UpdateUserRoleClassRequest()
        request.role.CopyFrom(UserRoleClass.raw_message)
        try:
            self.__stub.UpdateUserRoleClass(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def get(self, namespace: str, name: str) -> Union[UserRoleClass, bool]:
        request = GetUserRoleClassRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.GetUserRoleClass(request)
            return UserRoleClass(response.role, self)
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def delete(self, namespace: str, name: str) -> bool:
        request = DeleteUserRoleClassRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.DeleteUserRoleClass(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def list(self, namespace: str) -> Union[List[UserRoleClass], bool]:
        request = ListUserRoleClassesRequest()
        request.namespace = namespace
        try:
            response = self.__stub.ListUserRoleClasses(request)
            return [UserRoleClass(item, self) for item in response.roles.items]
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False



"""Top-level package for modela-python-client."""

__author__ = """The Modela Authors"""
__email__ = 'support@metaprov.com'
__version__ = '1.0.0'

from modela.server import *
from modela.data.models import *
from modela.data.common import *
from modela.training.models import *
from modela.training.common import *
from modela.infra.models import *
from modela.infra.common import *
from modela.inference.models import *
from modela.inference.common import *


import asyncio
import os
import time

import grpc
import pandas
from github.com.metaprov.modelaapi.pkg.apis.training.v1alpha1.generated_pb2 import \
    ModelAutobuilder as MDModelAutobuilder
from github.com.metaprov.modelaapi.services.modelautobuilder.v1.modelautobuilder_pb2_grpc import \
    ModelAutobuilderServiceStub
from github.com.metaprov.modelaapi.services.modelautobuilder.v1.modelautobuilder_pb2 import \
    CreateModelAutobuilderRequest, \
    UpdateModelAutobuilderRequest, \
    DeleteModelAutobuilderRequest, GetModelAutobuilderRequest, ListModelAutobuildersRequest

from modela.data.Dataset import Dataset
from modela.data.DataSource import DataSource
from modela.inference.DataApp import DataApp
from modela.inference.Predictor import Predictor
from modela.training.Study import Study
from modela.training.models import *
from modela.data.models import SampleSettings
from modela.data.common import *
from modela.infra.VirtualBucket import VirtualBucket
from modela.infra.models import Workload
from modela.infra.Lab import Lab
from modela.infra.ServingSite import ServingSite
from modela.training.common import *
from modela.Resource import Resource
from modela.ModelaException import ModelaException
from modela.common import ObjectReference
from typing import List, Union
from modela.common import Metric



class ModelAutobuilder(Resource):
    def __init__(self, item: MDModelAutobuilder = MDModelAutobuilder(), client=None, namespace="", name="",
                 version=Resource.DefaultVersion,
                 lab: Union[ObjectReference, Lab, str] = "default-lab",
                 serving_site: Union[ObjectReference, ServingSite, str] = "default-serving-site",
                 task_type: TaskType = TaskType.BinaryClassification,
                 workload: Workload = Workload("general-large"),
                 bucket: Union[ObjectReference, VirtualBucket, str] = "default-minio-bucket",
                 dataframe: pandas.DataFrame = None,
                 data_file: str = None,
                 data_bytes: bytes = None,
                 target_column: str = None,
                 objective: Metric = Metric.Accuracy,
                 feature_selection: bool = False,
                 feature_engineering: bool = False,
                 max_models: int = 1,
                 max_time: int = 512,
                 trainers: int = 1,
                 predictor_access_type: AccessType = AccessType.ClusterIP,
                 predictor_autoscale: bool = False,
                 create_data_app: bool = False):
        """
        :param client: The Study client repository, which can be obtained through an instance of Modela.
        :param namespace: The target namespace of the resource.
        :param name: The name of the resource.
        :param version: The version of the resource.
        :param lab: The object reference, Lab object, or lab name under the tenant of the resource for which all
            data science workloads will be performed under.
        :param serving_site: The object reference, Serving Site object, or name under the tenant of the resource for which
            the created Predictor will be deployed under.
        :param task_type: The target task type in relation to the data being used.
        :param workload: The workload specification which determines the resources which will be allocated for training
            and serving workloads.
        :param bucket: The bucket which the raw dataset data will be uploaded to.
        :param dataframe: The Pandas Dataframe will be serialized and uploaded to create a new Dataset resource.
        :param data_file: The file path which will be read and uploaded to create a new Dataset resource.
        :param data_bytes: The raw data as bytes that will be uploaded to create a new Dataset resource.
        :param target_column: The target column of the Dataset to be created.
        :param objective: The objective metric to be used when determining the best model from the Study.
        :param feature_selection: If True feature selection will be performed on the Dataset before the model search.
        :param feature_engineering: If True feature engineering will be performed on the Dataset before the model search.
        :param max_models: The maximum number of models to sample before selecting the best model.
        :param max_time: The maximum amount of time in seconds which the model search can run for.
        :param trainers: The number of parallel trainers to allocate during the model search
        :param predictor_access_type: The access type of the Predictor to be created. See https://www.modela.ai/docs/docs/serving/production/
            for documentation on how different access types expose the Predictor service
        :param predictor_autoscale: If true the created Predictor will automatically scale for traffic
        :param create_data_app: If true a Data Application will be created to serve a live model dashboard
        """
        self.default_resource = False
        super().__init__(item, client, namespace=namespace, name=name, version=version)
        if not self.default_resource:  # Ignore the rest of the constructor; MAB is immutable
            return

        spec = self.spec
        if type(lab) == Lab:
            lab = lab.reference
        elif type(lab) == str:
            lab = ObjectReference(Namespace=client.modela.tenant, Name=lab)

        if type(serving_site) == ServingSite:
            serving_site = serving_site.reference
        elif type(serving_site) == str:
            serving_site = ObjectReference(Namespace=client.modela.tenant, Name=serving_site)

        spec.LabRef = lab
        spec.ServingSiteRef = serving_site

        if data_file is not None:
            with open(data_file, 'r') as f:
                data_bytes = f.read()
            data_file = os.path.basename(data_file)
        elif dataframe is not None:
            data_bytes = bytes(dataframe.to_csv(index=False), encoding='utf-8')

        spec.Location = client.modela.FileService.upload_file(data_file or name, data_bytes,
                                                              client.modela.tenant,
                                                              namespace, version, bucket, "datasets", name)

        datasource = client.modela.DataSource(namespace=namespace, name=f"{name}-sdk-autobuilder", version=version,
                                              infer_bytes=data_bytes, target_column=target_column, task_type=task_type,
                                              file_type=FlatFileType.Csv)
        spec.DataSourceSpec = datasource.spec
        spec.DataProductName = namespace
        spec.DataProductVersionName = version
        spec.DatasourceName = name
        spec.DatasetName = name
        spec.Task = task_type
        spec.TargetColumn = target_column

        spec.Resources = workload
        spec.Objective = objective
        spec.FeatureSelection = feature_selection
        spec.FeatureEngineering = feature_engineering
        spec.MaxModels = max_models
        spec.MaxTime = max_time
        spec.Trainers = trainers
        spec.AccessMethod = predictor_access_type
        spec.AutoScale = predictor_autoscale
        spec.Dataapp = create_data_app

    @property
    def spec(self) -> ModelAutobuilderSpec:
        return ModelAutobuilderSpec().copy_from(self._object.spec)

    @property
    def status(self) -> ModelAutobuilderStatus:
        self.sync()
        return ModelAutobuilderStatus().copy_from(self._object.status)

    def default(self):
        self.default_resource = True
        ModelAutobuilderSpec().apply_config(self._object.spec)

    @property
    def datasource(self) -> DataSource:
        """ Returns the Data Source associated with the resource """
        self.ensure_client_repository()
        if self.status.DataSourceName == "":
            raise ValueError("Model Autobuilder {0} has no Data Source.".format(self.name))
        return self._client.modela.DataSource(namespace=self.namespace, name=self.status.DataSourceName)

    @property
    def dataset(self) -> Dataset:
        """ Returns the Dataset associated with the resource """
        self.ensure_client_repository()
        if self.status.DatasetName == "":
            raise ValueError("Model Autobuilder {0} has no Dataset.".format(self.name))
        return self._client.modela.Dataset(namespace=self.namespace, name=self.status.DatasetName)

    @property
    def study(self) -> Study:
        """ Returns the Study associated with the resource """
        self.ensure_client_repository()
        if self.status.StudyName == "":
            raise ValueError("Model Autobuilder {0} has no Study.".format(self.name))
        return self._client.modela.Study(namespace=self.namespace, name=self.status.StudyName)

    @property
    def predictor(self) -> Predictor:
        """ Returns the Predictor associated with the resource """
        self.ensure_client_repository()
        if self.status.PredictorName == "":
            raise ValueError("Model Autobuilder {0} has no Predictor.".format(self.name))
        return self._client.modela.Predictor(namespace=self.namespace, name=self.status.PredictorName)

    @property
    def dataapp(self) -> DataApp:
        """ Returns the Data Application associated with the resource """
        self.ensure_client_repository()
        if self.status.DataSourceName == "":
            raise ValueError("Model Autobuilder {0} has no Data Application.".format(self.name))
        return self._client.modela.DataApp(namespace=self.namespace, name=self.status.PredictorName)

    def submit_and_visualize(self, replace=False, show_progress_bar=True):
        """
        Submit the resource and call visualize().

        :param replace: Replace the resource if it already exists on the cluster.
        :param show_progress_bar: If enabled, the visualization will render a progress bar indicating the study progress.
        """
        self.submit(replace)
        self.visualize(show_progress_bar)

    def visualize(self, show_progress_bar=True):
        """ Display a real-time visualization of the Study's progress

        :param show_progress_bar: If enabled, the visualization will render a progress bar.
         """
        while self.status.DatasetName == "":
            time.sleep(1 / 5)

        self.dataset.visualize()
        while self.status.StudyName == "":
            time.sleep(1 / 5)

        self.study.visualize(show_progress_bar)

    async def wait_until_phase(self, phase: ModelAutobuilderPhase):
        """ Returns a coroutine which blocks until the specified phase is reached, or the ModelAutoBuilder fails """
        while self.status.Phase not in (phase, ModelAutobuilderPhase.Failed):
            await asyncio.sleep(1 / 5)


class ModelAutobuilderClient:
    def __init__(self, stub, modela):
        self.modela = modela
        self.__stub: ModelAutobuilderServiceStub = stub

    def create(self, modelautobuilder: ModelAutobuilder) -> bool:
        request = CreateModelAutobuilderRequest()
        request.modelautobuilder.CopyFrom(modelautobuilder.raw_message)
        try:
            response = self.__stub.CreateModelAutobuilder(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def update(self, modelautobuilder: ModelAutobuilder) -> bool:
        request = UpdateModelAutobuilderRequest()
        request.modelautobuilder.CopyFrom(modelautobuilder.raw_message)
        try:
            self.__stub.UpdateModelAutobuilder(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def get(self, namespace: str, name: str) -> ModelAutobuilder:
        request = GetModelAutobuilderRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.GetModelAutobuilder(request)
            return ModelAutobuilder(response.modelautobuilder, self)
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)

    def delete(self, namespace: str, name: str) -> bool:
        request = DeleteModelAutobuilderRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.DeleteModelAutobuilder(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def list(self, namespace: str, labels: dict = None) -> List[ModelAutobuilder]:
        request = ListModelAutobuildersRequest()
        request.namespace = namespace
        if labels is not None:
            request.labels.update(labels)

        try:
            response = self.__stub.ListModelAutobuilders(request)
            return [ModelAutobuilder(item, self) for item in response.modelautobuilders.items]
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)



import grpc
from github.com.metaprov.modelaapi.pkg.apis.team.v1alpha1.generated_pb2 import PostMortem as MDPostMortem
from github.com.metaprov.modelaapi.services.postmortem.v1.postmortem_pb2_grpc import PostMortemServiceStub
from github.com.metaprov.modelaapi.services.postmortem.v1.postmortem_pb2 import CreatePostMortemRequest, \
    UpdatePostMortemRequest, \
    DeletePostMortemRequest, GetPostMortemRequest, ListPostMortemsRequest

from modela.Resource import Resource
from modela.ModelaException import ModelaException
from typing import List, Union


class PostMortem(Resource):
    def __init__(self, item: MDPostMortem = MDPostMortem(), client=None, namespace="", name=""):
        super().__init__(item, client, namespace=namespace, name=name)


class PostMortemClient:
    def __init__(self, stub, modela):
        self.modela = modela
        self.__stub: PostMortemServiceStub = stub

    def create(self, postmortem: PostMortem) -> bool:
        request = CreatePostMortemRequest()
        request.postmortem.CopyFrom(postmortem.raw_message)
        try:
            response = self.__stub.CreatePostMortem(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def update(self, postmortem: PostMortem) -> bool:
        request = UpdatePostMortemRequest()
        request.postmortem.CopyFrom(postmortem.raw_message)
        try:
            self.__stub.UpdatePostMortem(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def get(self, namespace: str, name: str) -> Union[PostMortem, bool]:
        request = GetPostMortemRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.GetPostMortem(request)
            return PostMortem(response.postmortem, self)
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def delete(self, namespace: str, name: str) -> bool:
        request = DeletePostMortemRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.DeletePostMortem(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def list(self, namespace: str) -> Union[List[PostMortem], bool]:
        request = ListPostMortemsRequest()
        request.namespace = namespace
        try:
            response = self.__stub.ListPostMortems(request)
            return [PostMortem(item, self) for item in response.postmortems.items]
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False





from dataclasses import dataclass
from enum import Enum
from typing import List
from dataclasses import dataclass, field

from github.com.metaprov.modelaapi.services.common.v1.common_pb2 import Plot
from k8s.io.api.core.v1.generated_pb2 import ObjectReference, SecretReference
from k8s.io.apimachinery.pkg.apis.meta.v1.generated_pb2 import Time

from github.com.metaprov.modelaapi.pkg.apis.catalog.v1alpha1.generated_pb2 import TestSuite, DataTestCase, TestSuiteResult, \
    DataTestCaseResult

from modela.Configuration import Configuration, ImmutableConfiguration, datamodel


class ConditionStatus(Enum):
    ConditionTrue = "True"
    ConditionFalse = "False"
    ConditionUnknown = "Unknown"


class TaskName(Enum):
    BatchPredictTask = "batch-predict"
    ForecastTask = "forecast"
    ProfileDatasetTask = "profile-dataset"
    SnapshotDatasetTask = "snapshot-dataset"
    ProfileStudyTask = "profile-study"
    ProfileModelTask = "profile-model"
    PublishModelTask = "publish-model"
    BakeModelTask = "bake-model"
    RunRecipeTask = "run-recipe"
    ReportDatasetTask = "report-dataset"
    ReportModelTask = "report-model"
    ReportStudyTask = "report-study"
    ReportPeriodTask = "report-period"
    ReportSummaryTask = "report-summary"
    SplitDatasetTask = "split-dataset"
    TestEnsembleTask = "test-ensemble"
    TestModelTask = "test-model"
    TrainEnsembleTask = "train-ensemble"
    TrainModelTask = "train-model"
    ValidateDatasetTask = "validate-dataset"
    MergeForecastTask = "merge-forecast"
    PartitionForecastTask = "partition-forecast"
    SplitDatasetToRungs = "split-dataset-to-rungs"
    CompileTask = "compile-model"
    PackageModelTask = "package-model"
    FeatureGenTask = "feature-gen"
    FeatureSelectTask = "feature-select"
    RunSqlQueryTask = "run-sql-query"
    RunWebRequestTask = "run-web-request"
    GenerateDataset = "generate-dataset"
    ValidateModel = "validate-model"


class StatusError(Enum):
    InvalidConfiguration = "InvalidConfiguration"
    InsufficientResources = "InsufficientResources"
    CreateError = "CreateError"
    UpdateError = "UpdateError"
    DeleteError = "DeleteError"


class AssertionType(Enum):
    MultiDatasetSameNumberOfRows = "multi-dataset-same-number-of-rows"
    MultiDatasetOuterJoinEmpty = "multi-dataset-outer-join-empty"
    MultiDatasetOuterJoinNotEmpty = "multi-dataset-outer-join-not-empty"
    MultiDatasetInnerJoinEmpty = "multi-dataset-inner-join-empty"
    MultiDatasetInnerJoinNotEmpty = "multi-dataset-inner-join-not-empty"
    MultiDatasetLeftJoinEmpty = "multi-dataset-left-join-empty"
    MultiDatasetLeftJoinNotEmpty = "multi-dataset-left-join-not-empty"
    MultiDatasetRightJoinEmpty = "multi-dataset-right-join-empty"
    MultiDatasetRightJoinNotEmpty = "multi-dataset-right-join-not-empty"
    DatasetColumnsCountEqual = "dataset-columns-count-equal"
    DatasetColumnsNameInSet = "dataset-columns-in-set"
    DatasetColumnsInOrderedList = "dataset-columns-in-ordered-list"
    DatasetRowCountBetween = "dataset-row-count-between"
    DatasetNotEmpty = "dataset-dataset-not-empty"
    DatasetTestNameNameEmpty = "dataset-empty"
    MultiColumnCorr = "multi-column-corr"
    ColumnTestNameColumnExist = "column-exist"
    ColumnHaveValues = "column-have-values"
    ColumnHasNoValue = "column-has-no-values"
    ColumnHaveNulls = "column-value-have-nulls"
    ColumnHasNoNull = "column-value-has-no-nulls"
    ColumnOfType = "column-of-type"
    ColumnValuesInSet = "column-values-in-set"
    ColumnValuesIncreasing = "column-values-increasing"
    ColumnsValuesDecreasing = "column-values-decreasing"
    ColumnValueLengthBetween = "column-value-length-between"
    ColumnValueNameMatchRegex = "column-value-match-regex"
    ColumnValueIsDate = "column-value-is-date"
    ColumnValueIsJson = "column-value-is-json"
    ColumnValueInDomain = "column-value-in-domain"
    ColumnUniqueValueCountBetween = "column-unique-value-count-between"
    ColumnOutlierValueUniqueBetween = "column-outlier-value-count-between"
    ColumnValidValueUniqueBetween = "column-valid-values-count-between"
    ColumnMismatchValueBetween = "column-mismatch-values-between"
    ColumnValueMinBetween = "column-value-min-between"
    ColumnValueLowerQuartileBetween = "column-value-lower-quartile-between"
    ColumnValueMedianBetween = "column-value-median-between"
    ColumnValueAvgBetween = "column-value-average-between"
    ColumnValueUpperQuartileBetween = "column-value-upper-quartile-between"
    ColumnValueMaxBetween = "column-value-max-between"
    ColumnValueStddevBetween = "column-value-stddev-between"
    ColumnValueChiSquarePValueBetween = "column-value-chi-square-p-value-between"
    ColumnValuePairCramersBetween = "column-value-pair-cramers-between"
    FileSizeBetween = "file-size-between"
    FileExist = "file-exist"
    FileRegexMatchCountBetween = "file-regex-match-count-between"
    FileValidJson = "file-valid-json"
    FileValidCsv = "file-valid-csv"

    # Model Tests
    ModelAccuracy     = "model-accuracy-greater-than"
    ModelRocAuc     = "model-roc-auc-greater-than"
    ModelF1     = "model-f1-greater-than"
    ModelPrecision     = "model-precision-greater-than"
    ModelRecall     = "model-recall-less-than"
    ModelMSE     = "model-mse-less-than"
    ModelRMSE     = "model-rmse-less-than"
    ModelMAPE     = "model-mape-less-than"

    ModelAccuracyGreaterThanBaseline      = "model-accuracy-greater-than-baseline"
    ModelRocAucGreaterThanBaseline     = "model-roc-auc-greater-than-baseline"
    ModelF1GreaterThanBaseline     = "model-f1-greater-than-baseline"
    ModelPrecisionGreaterThanBaseline     = "model-precision-greater-than-baseline"
    ModelRecallGreaterThanBaseline     = "model-recall-less-than-baseline"
    ModelMSELessThanBaseline     = "model-mse-less-than-baseline"
    ModelRMSELessThanBaseline     = "model-rmse-less-than-baseline"
    ModelMAPELessThanBaseline     = "model-mape-less-than-baseline"


class PriorityLevel(Enum):
    Low = "low"
    Medium = "medium"
    High = "high"
    Urgent = "urgent"


class HardwareTarget(Enum):
    AMDEPYC2 = "amd-epyc-2"
    ARMA53 = "arma-53"
    ARMA72 = "arma-72"
    IntelCascadeLake = "intel-cascade-lake"
    IntelBroadwell = "intel-broadwell"
    IntelSkylake = "intel-skylake"
    TeslaV100 = "tesla-v100"
    TeslaK80 = "tesla-k80"
    T4 = "t4"
    RaspberryPi8MODELB = "raspberry-pi8-modela-b"


class TriggerScheduleEventType(Enum):
    Now = "now"
    Once = "once"
    Hourly = "hourly"
    Daily = "daily"
    Weekly = "weekly"
    Monthly = "monthly"
    Yearly = "yearly"
    Cron = "cron"


class Freq(Enum):
    Seconds = "second"
    Minutes = "minute"
    Hours = "hour"
    Days = "day"
    Weeks = "week"
    Months = "month"
    Qtrs = "quarter"
    Years = "year"


@datamodel(proto=Time)
class Time(Configuration):
    Seconds: int = 0
    Nanos: int = 0


class Metric(Enum):
    Accuracy = "accuracy"
    AveragePrecision = "average-precision"
    BalancedAccuracy = "balanced-accuracy"
    F1Binary = "f1"
    F1Micro = "f1-micro"
    F1Macro = "f1-macro"
    F1Weighted = "f1-weighted"
    F1Samples = "f1-samples"
    PrecisionBinary = "precision"
    PrecisionMicro = "precision-micro"
    PrecisionMacro = "precision-macro"
    PrecisionWeighted = "precision-weighted"
    PrecisionSamples = "precision-samples"
    RecallBinary = "recall"
    RecallMacro = "recall-macro"
    RecallMicro = "recall-micro"
    RecallWeighted = "recall-weighted"
    RecallSamples = "recall-samples"
    LogLoss = "log-loss"
    RocAuc = "auc"
    RocAucMicro = "auc-micro"
    RocAucMacro = "auc-macro"
    RocAucWeighted = "auc-weighted"
    PRRocAuc = "pr-auc"
    PRRocAucMicro = "pr-auc-micro"
    PRRocAucMacro = "pr-auc-macro"
    PRRocAucWeighted = "pr-auc-weighted"
    ZeroOne = "zero-one"
    HammingLoss = "hamming-loss"
    HingeLoss = "hinge-loss"
    JacquardScore = "jacquard-loss"
    MatthewsCorrCoef = "mcc"
    Fp = "fp"
    Fn = "fn"
    Tn = "tn"
    Tp = "tp"
    Tpr = "tpr"
    Fpr = "fpr"
    Tnr = "tnr"
    MCC = "matthews-corr-coef"
    ExplainedVariance = "explained-variance"
    MaxError = "max-error"
    MAE = "mae"
    MSE = "mse"
    MSLE = "msle"
    RMSE = "rmse"
    RMSLE = "rmsle"
    MedianAbsoluteError = "median-absolute-error"
    R2 = "r2"
    AdjR2 = "adj-r2"
    MeanPoissonDeviance = "mean_poisson_deviance"
    MeanGammaDeviance = "mean-gamma-deviance"
    MeanTweedieDeviance = "mean-tweedie-deviance"
    MAPE = "mape"
    MAZE = "maze"
    MDAPE = "mdape"
    SMAPE = "smape"
    AdjustedMutualInfoScore = "adjusted-mutual-info-score"
    AdjustedRandScore = "adjusted-rand-score"
    CompletenessScore = "completeness-score"
    FowlkesMallowsScore = "fowlkes-mallows-score"
    HomogeneityScore = "homogeneity-score"
    MutualInfoScore = "mutual-info-score"
    NormalizedMutualInfoScore = "normalized-mutual-info-score"
    VMeasureScore = "v-measure-score"
    P50Latency = "p50-latency"
    P95Latency = "p95-latency"
    P99Latency = "p99-latency"
    Cpu = "cpu"
    Gpu = "gpu"
    Mem = "mem"
    GpuMem = "gpu-mem"
    ReqSec = "req-per-sec"
    UncrainPredictionPercent = "uncertain-prediction-percent"
    Null = "none"


@datamodel(proto=ObjectReference)
class ObjectReference(Configuration):
    """
    An Object Reference defines the location of a resource on the current operational cluster, denoted by the namespace
    of the resource and the name of the resource which exists on that namespace.
    """

    Namespace: str = ""
    Name: str = ""


@datamodel(proto=SecretReference)
class SecretReference(Configuration):
    Name: str = ""
    Namespace: str = ""


@datamodel(proto=Plot)
class Plot(ImmutableConfiguration):
    Fname: str = ""
    Img: bytes = ""
    Name: str = ""
    Title: str = ""
    Url: str = ""


@datamodel(proto=TestSuite)
class TestSuite(Configuration):
    Enabled: bool = False
    Tests: List[DataTestCase] = field(default_factory=lambda: [])


@datamodel(proto=DataTestCase)
class DataTestCase(Configuration):
    Enabled: bool = True
    AssertThat: AssertionType
    Metric: Metric
    Entity: str = ""
    ExpectedNumber: float
    ExpectedString: str
    Min: float
    Max: float
    ExpectedSet: List[str]
    StrictMin: bool
    StrictMax: bool
    Generated: bool
    DisplayName: str
    Tags: List[str]


@datamodel(proto=TestSuiteResult)
class TestSuiteResult(Configuration):
    Fixture: ObjectReference
    Failures: int = 0
    Errors: int = 0
    StartTime: Time
    StopTime: Time
    Tests: List[DataTestCaseResult]


@datamodel(proto=DataTestCaseResult)
class TestCaseResult(Configuration):
    Actual: Measurement
    Failure: bool = False
    Error: bool = False
    FailureMsg: str = ""



import grpc
from github.com.metaprov.modelaapi.pkg.apis.data.v1alpha1.generated_pb2 import LabelingPipeline as MDLabelingPipeline
from github.com.metaprov.modelaapi.services.labelingpipeline.v1.labelingpipeline_pb2_grpc import LabelingPipelineServiceStub
from github.com.metaprov.modelaapi.services.labelingpipeline.v1.labelingpipeline_pb2 import CreateLabelingPipelineRequest, \
    UpdateLabelingPipelineRequest, \
    DeleteLabelingPipelineRequest, GetLabelingPipelineRequest, ListLabelingPipelineRequest

from modela.Resource import Resource
from modela.ModelaException import ModelaException
from typing import List, Union


class LabelingPipeline(Resource):
    def __init__(self, item: MDLabelingPipeline = MDLabelingPipeline(), client=None, namespace="", name="", version=Resource.DefaultVersion):
        super().__init__(item, client, namespace=namespace, name=name, version=version)


class LabelingPipelineClient:
    def __init__(self, stub, modela):
        self.modela = modela
        self.__stub: LabelingPipelineServiceStub = stub

    def create(self, labelingpipeline: LabelingPipeline) -> bool:
        request = CreateLabelingPipelineRequest()
        request.labelingpipeline.CopyFrom(labelingpipeline.raw_message)
        try:
            response = self.__stub.CreateLabelingPipeline(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def update(self, labelingpipeline: LabelingPipeline) -> bool:
        request = UpdateLabelingPipelineRequest()
        request.labelingpipeline.CopyFrom(labelingpipeline.raw_message)
        try:
            self.__stub.UpdateLabelingPipeline(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def get(self, namespace: str, name: str) -> Union[LabelingPipeline, bool]:
        request = GetLabelingPipelineRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.GetLabelingPipeline(request)
            return LabelingPipeline(response.labelingpipeline, self)
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def delete(self, namespace: str, name: str) -> bool:
        request = DeleteLabelingPipelineRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.DeleteLabelingPipeline(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def list(self, namespace: str) -> Union[List[LabelingPipeline], bool]:
        request = ListLabelingPipelineRequest()
        request.namespace = namespace
        try:
            response = self.__stub.ListLabelingPipelines(request)
            return [LabelingPipeline(item, self) for item in response.labelingpipelines.items]
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False





from enum import Enum


class HolidayCountry(Enum):
    UnitedState = "US"
    Brazil = "BR"
    Indonesia = "ID"
    India = "IN"
    Malaysia = "MY"
    Vietnam = "VN"
    Thailand = "TH"
    Philippines = "PH"
    Pakistan = "PK"
    Bangladesh = "BD"
    Egypt = "EG"
    China = "CN"
    Russian = "RU"
    Korea = "KR"
    Belarus = "BY"
    UnitedArabEmirates = "AE"
    NoneHoliday = "none"


class Frequency(Enum):
    Seconds = "second"
    Minutes = "minute"
    Hours = "hour"
    Days = "day"
    Weeks = "week"
    Months = "month"
    Qtrs = "quarter"
    Years = "year"


class TaskType(Enum):
    BinaryClassification = "binary-classification"
    MultiClassification = "multi-classification"
    MultiLabelClassification = "multi-label-classification"
    Forecasting = "forecasting"
    Regression = "regression"
    Clustering = "clustering"
    Recommendation = "recommendation"
    OutlierDetection = "outlier-detection"
    NoveltyDetection = "novelty-detection"
    TopicModeling = "topic-modeling"
    VideoActionRecognition = "video-action-recognition"
    VideoClassification = "video-classification"
    VideoObjectTracking = "video-object-tracking"
    ImageClassification = "image-classification"
    ImageMultiLabelClassification = "image-multi-classification"
    ImageObjectDetection = "image-object-detection"
    ImageSegmentation = "image-segmentation"
    AutoDetectTask = "auto"


class SubtaskType(Enum):
    TextClassification = "text-classification"
    TextMultiClassification = "text-multi-classification"
    TextRegression = "text-regression"
    TextMultiLabelClassification = "text-multi-label-classification"
    TextConversation = "text-conversation"
    TextLangGeneration = "text-lang-generation"
    TextLangModel = "text-lang-model"
    TextMultiModalClassification = "text-multi-modal"
    TextNER = "text-ner"
    TextQA = "text-qa"
    TextSummarization = "text-summarization"
    TextSentencePairClassification = "text-sentence-pair"
    TextRepresentationGeneration = "text-representation-generation"
    TextSentimentAnalysis = "text-sentiment-analysis"
    TextCodeGeneration = "text-code-generation"
    TextTranslation = "text-translation"
    TextLangDetection = "text-lang-detection"
    TextGrammarCorrection = "text-grammar-correction"
    TextParaphrasing = "text-paraphrasing"
    TextIntentClassification = "text-intent-classification"
    TextSemanticSimilarity = "text-semantic-similarity"
    TextKeywordExtraction = "text-keyword-extraction"
    TextPOS = "text-pos"
    TextTokenization = "text-tokenization"
    TextLemmalization = "text-lemma"

    ImageClassification = "image-classification"
    ImageMultiLabelClassification = "image-multi-label-classification"
    ImageObjectDetection = "image-object-detection"
    ImageSegmentation = "image-segmentation"

    VideoActionRecognition = "video-action-recognition"
    VideoClassification = "video-classification"
    VideoObjectTracking = "video-object-tracking"

    NoneSubtask = "none"



class Imputation(Enum):
    RemoveRowsWithMissingValues = "remove-rows-with-missing-values"
    ReplaceWithMean = "replace-with-mean"
    ReplaceWithMedian = "replace-with-median"
    ReplaceWithArbitraryValue = "replace-with-arbitrary-value"
    ReplaceWithEndOfTail = "replace-with-end-of-tail"
    ReplaceWithRandomSample = "replace-with-random-sample"
    FreqCategory = "freq-category-imputation"
    AddMissingValueIndicator = "add-missing-value-indicator"
    Knn = "knn"
    Iterative = "iterative"
    MICE = "mice"
    No = "no-imputation"
    AutoImputer = "auto"


class Scaling(Enum):
    Standard = "standard-scaling"
    MaxAbs = "max-abs-scaling"
    MinMax = "min-max-scaling"
    Normalization = "normalizion-scaling"
    Robust = "robust-scaling"
    ScaleToUnitNorm = "scale-to-unit-norm"
    NoScaling = "none"
    Auto = "auto"


class DatasetType(Enum):
    Tabular = "tabular"
    Image = "image"
    Text = "text"
    Video = "video"
    Audio = "audio"


class RelationshipArity(Enum):
    OneToOne = "one-to-one"
    OneToMany = "one-to-many"
    ManyToMany = "many-to-many"


class FeatureSelectionType(Enum):
    DropFeatures = "drop-features"
    DropConstantFeatures = "drop-constant-features"
    DropDuplicateFeatures = "drop-deplicated-features"
    DropCorrelatedFeatures = "drop-correlated-features"
    MutualInformation = "mutal-information"
    ChiSquare = "chisquare-fearture-selection"
    StepForward = "step-forward-feature-selection"
    StepBackward = "step-backward-feature-selection"
    LassoRegression = "lasso-regression-feature-selection"
    TreeImportance = "tree-importance-feature-selection"
    RecursiveFeatureElimination = "recursive-feature-elimination"
    RecursiveFeatureAddition = "recursive-feature-addition"
    SelectPercentile = "select-percentile"
    SelectKBest = "select-kbest"
    SelectFpr = "select-fpr"
    SelectFdr = "select-fdr"
    VarianceThreshold = "variance-threshold"
    Nothing = "none"
    Auto = "auto"


class ShapType(Enum):
    Permutation = "permutation"
    Partition = "partition"
    Tree = "tree"
    Kernel = "kernel"
    Sampling = "sampling"
    Linear = "linear"
    Deep = "deep"
    Gradient = "gradient"
    Auto = "auto"
    Unknown = "none"


class ModelClassType(Enum):
    FE = "feature-engineering"
    Baseine = "baseline"
    Search = "search"
    Ensemble = "ensemble"
    Test = "test"


class ModelType(Enum):
    Classical = "classical"
    DNN = "dnn"
    Transformer = "transformer"
    Chatbot = "chatbot"
    RL = "rl"


class CategoricalEncoding(Enum):
    OneHotEncoding = "one-hot-encoding"
    OneHotEncoderTop = "one-hot-encoding-top-categories"
    OrdinalEncoding = "ordinal-encoding"
    CountEncoding = "count-encoding"
    TargetEncoding = "target-encoding"
    WoERatioEncoding = "weight-of-evidence-encoding"
    BinaryEncoding = "binary-encoding"
    LabelEncoding = "label-encoding"
    HashEncoding = "hash-encoding"
    CatboostEncoding = "catboost-encoding"
    LeaveOneOutEncoding = "loo-encoding"
    NoEncoding = "no-encoding"
    AutoEncoding = "auto"
    NoneEncoding = "none"


class Discretisation(Enum):
    EqualWidth = "equal-width-discretisation-"
    EqualFreq = "equal-freq-discretisation"
    KBins = "kbin-discretisation"
    KernelCenterer = "kernel-centerer-discretisation"
    LabelBinarizer = "label-binarizer-discretisation"
    MultiLabelBinarizer = "multi-label-binarizer-discretisation"
    NoDiscretisation = "none"
    Auto = "auto"


class OutlierHandling(Enum):
    Trim = "trim-outliers"
    Winsorizer = "winsorizer-outliers"
    Cap = "cap-outliers"
    ZeroCode = "zero-code-outliers"
    NoneOutlier = "none"
    AutoOutlier = "auto"


class VariableTransformation(Enum):
    LogTransformation = "log-transformation"
    ReciprocalTransformation = "reciprocal-transformation"
    SqrtTransformation = "sqrt-transformation"
    PowerTransformation = "power-transformation"
    BoxCoxTransformation = "box-cox-transformation"
    YeoJohnsonTransformation = "yj-transformation"
    NoneTransform = "none"
    AutoTransform = "auto"


class DatetimeTransformation(Enum):
    ExtractDateTimeInformation = "extract-datetime-information"
    NoneDatetime = "none"
    AutoDatetime = "auto"


class TextEncoding(Enum):
    TfIdf = "tfidf"
    CountVectorizer = "count-vec"
    HashingVectorizer = "hashing-vec"
    NoEncoding = "none"
    Auto = "auto"


class CvType(Enum):
    Nothing = "none"
    GroupKFold = "group-kfold"
    GroupShuffleSplit = "group-shuffle-split"
    CVTypeKFold = "kfold"
    CVTypeLeaveOneGroupOut = "leave-one-group-out"
    LeavePGroupsOut = "leave-p-groups-out"
    LeaveOneOut = "leave-one-out"
    LeavePOut = "leave-p-out"
    PredefinedSplit = "predefined-split"
    RepeatedKFold = "repeated-kfold"
    RepeatedStratifiedKFold = "repeated-stratified-k-fold"
    ShuffleSplit = "shuffle-split"
    StratifiedKFold = "stratified-k-fold"
    StratifiedShuffleSplit = "stratified-shuffle-split"
    StratifiedGroupKFold = "stratified-group-k-fold"
    TimeSeriesSplit = "time-series-split"


class EnsembleType(Enum):
    Voting = "voting"
    Stack = "stacking"


class DataSplitMethod(Enum):
    RandomStratified = "random-stratified"
    Random = "random"
    SplitColumn = "split-column"
    Time = "time"
    Auto = "auto"
    TestDataset = "test-dataset"


class Operation(Enum):
    LT = "lt"
    EQ = "eq"
    GT = "gt"
    NE = "ne"
    LE = "le"
    GE = "ge"


class ImbalanceHandling(Enum):
    ADASYN = "adasyn"
    BorderlineSMOTE = "baseline-smote"
    KMeansSMOTE = "kmean-smote"
    RandomOverSampler = "random-over-sampler"
    SMOTE = "smote"
    SMOTENC = "smotenc"
    SVMSMOTE = "svmsmote"
    ClassWeights = "class-weights"
    ImbalanceNone = "none"
    ImbalanceAuto = "auto"


class ModalityType(Enum):
    Data = "data"
    Epochs = "epochs"


class ModelPhase(Enum):
    Failed = "Failed"
    Pending = "Pending"
    Training = "Training"
    Trained = "Trained"
    Testing = "Testing"
    Tested = "Tested"
    Reporting = "Reporting"
    Reported = "Reported"
    Completed = "Completed"
    Publishing = "Publishing"
    Published = "Published"
    Packaging = "Packaging"
    Packaged = "Packaged"
    Profiling = "Profiling"
    Profiled = "Profiled"
    Explaining = "Explaining"
    Explained = "Explained"
    Aborted = "Aborted"
    Forecasting = "Forecasting"
    Forecasted = "Forecasted"
    Uat = "UAT"
    Deployed = "Deployed"
    Releasing = "Releasing"
    Live = "Live"
    Maintenance = "Maintenance"
    Retired = "Retired"


class ModelConditionType(Enum):
    ModelInitialized = "Initialized"
    ModelTrained = "Trained"
    ModelTested = "Tested"
    ModelReported = "Reported"
    ModelPackaged = "Packaged"
    ModelPublished = "Published"
    ModelProfiled = "Profiled"
    ModelReady = "Ready"
    ModelSaved = "Saved"
    ModelArchived = "Archived"
    ModelExplained = "Explained"
    ModelReleased = "Released"
    ModelAborted = "Aborted"
    ModelPaused = "Paused"
    ModelForecasted = "Forecasted"
    ModelUat = "UAT"
    ModelLive = "Live"
    ModelStale = "Stale"
    ModelMaintenance = "Maintenance"
    ModelRetired = "Retired"


class ClassicEstimator(Enum):
    KNeighborsRegressor = "knn-regressor"
    RidgeRegressor = "ridge-regressor"
    LassoRegressor = "lasso-regressor"
    AdaBoostRegressor = "ada-boost-regressor"
    DecisionTreeRegressor = "decision-tree-regressor"
    ExtraTreeRegressor = "extra-tree-regressor"
    LinearSvr = "linear-svr"
    Svr = "svr"
    PassiveAggressiveRegressor = "passive-agressive-regressor"
    SGDRegressor = "sgd-regressor"
    GradientBoostingRegressor = "gradient-boosting-regressor"
    HistGradientBoostingRegressor = "hist-gradient-boosting-regressor"
    RandomForestRegressor = "random-forest-regressor"
    XGBRegressor = "xgb-regressor"
    CatBoostRegressor = "catboost-regressor"
    LightGbmRegressor = "lightgbm-regressor"
    ElasticNetRegressor = "elasticnet-regressor"
    DNNRegressor = "dnn-regressor"
    EllipticEnvelope = "elliptic-envelope"
    OneClassSVM = "one-class-svm"
    IsolationForest = "isolation-forest"
    LocalOutlierFactor = "local-outlier-factor"
    Prophet = "prophet"
    ARIMA = "arima"
    AutoARIMA = "auto-arima"
    VRIMA = "vrima"
    ExponentialSmoothing = "exponential-smoothing"
    FastFourierTransform = "fast-fourier-transform"
    NBeats = "nbeats"
    ThetaMethod = "theata-method"
    ALS = "als"
    BayesianPersonalizedRanking = "bayesian-personalized-ranking"
    KNeighborsClassifier = "knn-classifier"
    AdaBoostClassifier = "ada-boost-classifier"
    BernoulliNB = "bernoulli-nb"
    DecisionTreeClassifier = "decision-tree-classifier"
    ExtraTreeClassifier = "extra-tree-classifier"
    LinearSVC = "linear-svc"
    SVC = "svc"
    PassiveAggressiveClassifier = "passive-aggressive-classifier"
    SGDClassifier = "sgd-classifier"
    LogisticRegression = "logistic-regression"
    GradientBoostingClassifier = "gradient-boosting-classifier"
    HistGradientBoostingClassifier = "hist-gradient-boosting-classifier"
    RandomForestClassifier = "random-forest-classifier"
    XGBClassifier = "xgboost-classifier"
    RidgeClassifier = "ridge-classifier"
    QuadraticDiscriminant = "quadratic-discriminant"
    LinearDiscriminant = "linear-discriminant"
    LightGBMClassifier = "lightgbm-classifier"
    CatBoostClassifier = "catboost-classifier"
    StackingEnsemble = "stacking-ensemble"
    UnknownEstimatorName = "unknown"
    NoneEstimatorName = "none"
    SpectralClustering = "spectral"
    AgglomerativeClustering = "agglomerative"
    GaussianMixtureClustering = "gaussian-mixture"
    KMeanClustering = "kmean"
    DBSCANClustering = "dbscan"
    VotingClassifier = "voting-classifier"
    VotingRegressor = "voting-regressor"
    StackingClassifier = "stacking-classifier"
    StackingRegressor = "stacking-regressor"


class Sampler(Enum):
    RandomSearch = "random"
    GridSearch = "grid"
    BayesianSearch = "bayesian"
    TPESearch = "tpe"
    ManualSearch = "manual"
    AutoSearchMethod = "auto"


class Pruner(Enum):
    NonePruner = "none"
    PatientPruner = "patient"
    MedianPruner = "median"
    PercentilePruner = "percentile"
    SHPruner = "sh"
    HyperbandPruner = "hyperband"
    ThresholdPruner = "threshold"


class AlgorithmFilter(Enum):
    Quick = "quick"
    LinearOnly = "linear-only"
    TreeOnly = "tree"
    DefaultParameters = "default-hp"
    NoFilter = "none"


class StudyPhase(Enum):
    ModelPending = "Pending"
    Splitting = "Spliting"
    Splitted = "Splitted"
    EngineeringFeature = "EngineeringFeatures"
    FeatureEngineered = "FeaturesEngineered"
    Baseline = "Baselining"
    Baselined = "Baselined"
    Searching = "Searching"
    Searched = "Searched"
    CreatingEnsembles = "CreatingEnsembles"
    CreatedEnsembles = "CreatedEnsembles"
    Testing = "Testing"
    Tested = "Tested"
    Reported = "Reported"
    Reporting = "Reporting"
    Profiling = "Profiling"
    Profiled = "Profiled"
    Explaining = "Explaining"
    Explained = "Explained"
    Completed = "Completed"
    Failed = "Failed"
    Aborted = "Aborted"
    Paused = "Paused"


StudyPhaseToProgress = {
    StudyPhase.ModelPending: "Pending",
    StudyPhase.Splitting: "Splitting Dataset",
    StudyPhase.Splitted: "Split Complete",
    StudyPhase.EngineeringFeature: "Feature Engineering Search",
    StudyPhase.FeatureEngineered: "Feature Engineering Complete",
    StudyPhase.Baseline: "Creating Baseline Models",
    StudyPhase.Baselined: "Baseline Models Complete",
    StudyPhase.Searching: "Searching Models",
    StudyPhase.Searched: "Search Complete",
    StudyPhase.CreatingEnsembles: "Creating Ensembles",
    StudyPhase.CreatedEnsembles: "Ensembles Created",
    StudyPhase.Testing: "Testing",
    StudyPhase.Tested: "Tested",
    StudyPhase.Reported: "Reported",
    StudyPhase.Reporting: "Reporting",
    StudyPhase.Profiling: "Profiling",
    StudyPhase.Profiled: "Profiled",
    StudyPhase.Explaining: "Explaining",
    StudyPhase.Explained: "Explained",
    StudyPhase.Completed: "Completed",
    StudyPhase.Failed: "Failed",
    StudyPhase.Aborted: "Aborted",
    StudyPhase.Paused: "Paused"
}


class StudyConditionType(Enum):
    StudyInitialized = "Initialized"
    StudySplitted = "StudySplitted"
    StudyFeatureEngineered = "StudyFeaturesEngineered"
    StudyBaselined = "StudyBaselined"
    StudySearched = "StudySearched"
    StudyEnsembleCreated = "ModelsEnsembleCreated"
    StudyTested = "ModelTested"
    StudyReported = "Reported"
    StudyProfiled = "Profiled"
    StudyExplained = "Explained"
    StudyAborted = "Aborted"
    StudyPaused = "Paused"
    StudySaved = "Saved"
    StudyCompleted = "Completed"
    StudyPartitioned = "Partitioned"


class TriggerType(Enum):
    OnDemand = "on-demand"
    Schedule = "on-schedule"
    NewData = "on-new-data"
    GithubEvent = "on-github-event"
    ConceptDrift = "on-concept-drift"
    PrefDegradation = "on-perf-degradation"


class ReportConditionType(Enum):
    ReportReady = "Ready"
    ReportSent = "Sent"
    ReportSaved = "Saved"


class ReportPhase(Enum):
    Pending = "Pending"
    Running = "Running"
    Completed = "Completed"
    Failed = "IsFailed"


class ReportFormat(Enum):
    Pdf = "pdf"


class ReportType(Enum):
    BinaryClassificationModelReport = "binary-classification-model"
    ForecastModelReport = "forecast-model"
    RegressionModelReport = "regression-model"
    MultiClassificationModelReport = "multi-classification-model"
    TextClassificationModelReport = "text-classification-model"
    ClassificationDatasetReport = "classification-dataset"
    ForecastDatasetReport = "forecast-dataset"
    TextClassificationDatasetReport = "text-classification-dataset"
    RegressionDatasetReport = "regression-dataset"
    SummaryReport = "summary-report"
    CustomReport = "custom-report"
    StudyReport = "study-report"
    ForecastReport = "forecast-report"
    FeatureReport = "feature-report"
    InvalidReport = "invalid-report"


class ModelAutobuilderConditionType(Enum):
    Ready = "Ready"
    DataProductReady = "DataProductReady"
    DataProductVersionReady = "DataProductVersionReady"
    DataSourceReady = "DataSourceReady"
    DatasetCompleted = "DatasetCompleted"
    StudyCompleted = "StudyCompleted"
    PredictorReady = "PredictorReady"
    DataAppReady = "DataAppReady"
    Saved = "Saved"


class ModelAutobuilderPhase(Enum):
    Pending = "Pending"
    DataSourceRunning = "DataSourceRunning"
    DataSourceReady = "DataSourceReady"
    DatasetRunning = "DatasetRunning"
    DatasetCompleted = "DatasetCompleted"
    StudyRunning = "StudyRunning"
    StudyCompleted = "StudyCompleted"
    PredictorRunning = "PredictorRunning"
    PredictorCompleted = "PredictorCompleted"
    DataAppRunning = "DataAppRunning"
    DataAppCompleted = "DataAppCompleted"
    Completed = "Completed"
    Aborted = "Aborted"
    Failed = "Failed"


class ModelTestName(Enum):
    ModelTest = "model-test"
    PredictionCountDrift = "prediction-count-drift"
    PredictionLatencyDrift = "prediction-latency-drift"
    PredictionLatencySkew = "prediction-latency-skew"
    ModelPerfSkew = "model-pref-skew"
    ModelPerfDrift = "model-pref-drift"
    CategoricalColumnSkew = "cat-column-skew"
    CategoricalColumnDrift = "cat-column-drift"
    NumericalColumnSkew = "numerical-column-skew"
    NumericalColumnDrift = "numerical-column-drift"
    MissingValueDrift = "missing-values-drift"
    MissingValueSkew = "missing-values-skew"
    ColumnStatSkew = "column-stat-skew"
    ColumnStatDrift = "column-stat-drift"


class Aggregate(Enum):
    Min = "min"
    Max = "max"
    Avg = "avg"
    Median = "median"
    Stddev = "stddev"
    Var = "var"


class ModelServingFormat(Enum):
    Protobuf = "protobuf"
    Pickle = "pickle"
    CloudPickle = "cloudpickle"
    MLLeap = "mlleap"
    MLModel = "mlmodel"
    H5 = "h5"
    Onyx = "onyx"
    Pmml = "pmml"
    TorchScript = "pt"



import grpc
from github.com.metaprov.modelaapi.pkg.apis.data.v1alpha1.generated_pb2 import DataPipeline as MDDataPipeline
from github.com.metaprov.modelaapi.services.datapipeline.v1.datapipeline_pb2_grpc import DataPipelineServiceStub
from github.com.metaprov.modelaapi.services.datapipeline.v1.datapipeline_pb2 import CreateDataPipelineRequest, \
    UpdateDataPipelineRequest, \
    DeleteDataPipelineRequest, GetDataPipelineRequest, ListDataPipelinesRequest

from modela.Resource import Resource
from modela.ModelaException import ModelaException
from typing import List, Union


class DataPipeline(Resource):
    def __init__(self, item: MDDataPipeline = MDDataPipeline(), client=None, namespace="", name="", version=Resource.DefaultVersion):
        super().__init__(item, client, namespace=namespace, name=name, version=version)


class DataPipelineClient:
    def __init__(self, stub, modela):
        self.modela = modela
        self.__stub: DataPipelineServiceStub = stub

    def create(self, datapipeline: DataPipeline) -> bool:
        request = CreateDataPipelineRequest()
        request.datapipeline.CopyFrom(datapipeline.raw_message)
        try:
            response = self.__stub.CreateDataPipeline(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def update(self, datapipeline: DataPipeline) -> bool:
        request = UpdateDataPipelineRequest()
        request.datapipeline.CopyFrom(datapipeline.raw_message)
        try:
            self.__stub.UpdateDataPipeline(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def get(self, namespace: str, name: str) -> Union[DataPipeline, bool]:
        request = GetDataPipelineRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.GetDataPipeline(request)
            return DataPipeline(response.datapipeline, self)
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def delete(self, namespace: str, name: str) -> bool:
        request = DeleteDataPipelineRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.DeleteDataPipeline(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def list(self, namespace: str) -> Union[List[DataPipeline], bool]:
        request = ListDataPipelinesRequest()
        request.namespace = namespace
        try:
            response = self.__stub.ListDataPipelines(request)
            return [DataPipeline(item, self) for item in response.datapipelines.items]
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False





from enum import Enum


class AccessType(Enum):
    ClusterIP = "cluster-ip"
    NodePort = "node-port"
    LoadBalancer = "load-balancer"
    Ingress = "ingress"
    Mesh = "mesh"
    Nothing = "none"


class PredictorConditionType(Enum):
    PredictorReady = "Ready"
    PredictorSaved = "Saved"


class ModelRole(Enum):
    Champion = "champion"
    Challenger = "challenger"
    Shadow = "shadow"


class ModelDeploymentPhase(Enum):
    Deploying = "Deploying"
    Deployed = "Deployed"
    Shadowing = "Shadowing"
    Releasing = "Releasing"
    Released = "Released"
    Failed = "Failed"


class PredictorType(Enum):
    Online = "online"
    Batch = "batch"
    Streaming = "streaming"





import grpc
from github.com.metaprov.modelaapi.pkg.apis.infra.v1alpha1.generated_pb2 import Alert as MDAlert
from github.com.metaprov.modelaapi.services.alert.v1.alert_pb2_grpc import AlertServiceStub
from github.com.metaprov.modelaapi.services.alert.v1.alert_pb2 import CreateAlertRequest, \
    UpdateAlertRequest, \
    DeleteAlertRequest, GetAlertRequest, ListAlertsRequest

from modela.Resource import Resource
from modela.ModelaException import ModelaException
from typing import List, Union


class Alert(Resource):
    def __init__(self, item: MDAlert = MDAlert(), client=None, namespace="", name=""):
        super().__init__(item, client, namespace=namespace, name=name)


class AlertClient:
    def __init__(self, stub, modela):
        self.modela = modela
        self.__stub: AlertServiceStub = stub

    def create(self, alert: Alert) -> bool:
        request = CreateAlertRequest()
        request.alert.CopyFrom(alert.raw_message)
        try:
            response = self.__stub.CreateAlert(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def update(self, alert: Alert) -> bool:
        request = UpdateAlertRequest()
        request.alert.CopyFrom(alert.raw_message)
        try:
            self.__stub.UpdateAlert(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def get(self, namespace: str, name: str) -> Union[Alert, bool]:
        request = GetAlertRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.GetAlert(request)
            return Alert(response.alert, self)
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def delete(self, namespace: str, name: str) -> bool:
        request = DeleteAlertRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.DeleteAlert(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def list(self, namespace: str) -> Union[List[Alert], bool]:
        request = ListAlertsRequest()
        request.namespace = namespace
        try:
            response = self.__stub.ListAlerts(request)
            return [Alert(item, self) for item in response.alerts.items]
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False





from enum import Enum


class CompilerType(Enum):
    TVM = "tvm"
    Onyx = "onyx"
    Xla = "xla"
    Nothing = "none"


class FileEncoding(Enum):
    Utf8 = "utf-8"
    Lating1 = "latin-1"
    Utf16 = "utf-16"


class Delimiter(Enum):
    CRLF = "crlf"
    CR = "cr"
    LF = "lf"
    Semicolon = "semicolon"
    Colon = "colon"
    Comma = "comma"
    Tab = "tab"
    Space = "space"
    Pipe = "pipe"


class EscapeCharacter(Enum):
    SingleQuote = "single-quote"
    DoubleQuote = "double-quote"
    Tilda = "tilda"
    NoneChar = "none"


class QuoteCharacter(Enum):
    SingleQuote = "single-quote"
    DoubleQuote = "double-quote"


class CompressionType(Enum):
    Tar = "tar"
    Gzip = "gzip"
    Zip = "zip"
    Uncompressed = "none"


class FlatFileType(Enum):
    Csv = "csv"
    Table = "tsv"
    Excel = "excel"
    Fwf = "fwf"
    Hdf5 = "hdf"
    Html = "html"
    Json = "json"
    Pickle = "pickle"
    Sas = "sas"
    Stata = "stata"
    Feather = "feather"
    Parquet = "parquet"


class DataType(Enum):
    Boolean = "boolean"
    DateTime = "datetime"
    Number = "number"
    Categorical = "categorical"
    Ordinal = "ordinal"
    Text = "text"
    Json = "json"
    NumberList = "number-list"
    CategoricalList = "categorical-list"
    TextList = "text-list"


class DataDomain(Enum):
    Url = "url"
    Path = "path"
    Password = "password"
    Time = "time"
    Count = "count"
    Interval = "interval"
    Nominal = "nominal"
    Email = "email"
    CreditCard = "creditCard"
    Uuid3 = "uuid"
    Uuid5 = "uuid5"
    Uuid = "uuid"
    Base64 = "base64"
    Latitude = "latitude"
    Longtitude = "longtitude"
    DnsName = "dns"
    Ip4 = "ip4"
    Ip6 = "ip6"
    Ssn = "ssn"
    Alpha = "alpha"
    AlphaNumeric = "alphanumeric"
    Unknown = "unknown"
    Color = "color"
    Currency = "currency"
    Date = "date"
    Guid = "guid"
    Hyperlink = "hyperlink"
    Image = "image"
    Media = "media"
    File = "file"
    Embedding = "embedding"
    Record = "record"
    Useless = "useless"
    Nothing = "none"


class SamplingType(Enum):
    Header = "header"
    Random = "random"
    Filter = "filter"
    Anomaly = "anomaly"
    Stratified = "stratified"


class DataLocationType(Enum):
    ObjectStorage = "object"
    SQLTable = "table"
    SQLView = "view"
    Stream = "stream"
    WebApi = "web"
    PublicDataset = "public-dataset"
    Dataset = "dataset"


class DatasetCondition(Enum):
    Reported = "Reported"
    Validated = "Validated"
    Snapshotted = "Snapshotted"
    Profiled = "Profiled"
    Ingested = "Ingested"
    Generated = "Generated"
    Saved = "Saved"
    Archived = "Archived"
    Ready = "Ready"


class DatasetPhase(Enum):
    Pending = "Pending"
    Generating = "Generating"
    GenSuccess = "GenSuccess"
    IngestRunning = "Ingesting"
    IngestSuccess = "Ingested"
    ReportRunning = "Reporting"
    ReportSuccess = "Reported"
    ProfileRunning = "Profiling"
    ProfileSuccess = "Profiled"
    ValidationRunning = "Validating"
    ValidationSuccess = "Validated"
    SnapshotRunning = "TakingSnapshot"
    SnapshotSuccess = "Snapshotted"
    Failed = "Failed"
    Aborted = "Aborted"
    Ready = "Ready"


class Color(Enum):
    Aliceblue = "aliceblue"
    Antiquewhite = "antiquewhite"
    Aqua = "aqua"
    Aquamarine = "aquamarine"
    Azure = "azure"
    Beige = "beige"
    Bisque = "bisque"
    Black = "black"
    Blanchedalmond = "blanchedalmond"
    Blue = "blue"
    Blueviolet = "blueviolet"
    Brown = "brown"
    Burlywood = "burlywood"
    Cadetblue = "cadetblue"
    Chartreuse = "chartreuse"
    Chocolate = "chocolate"
    Coral = "coral"
    Cornflowerblue = "cornflowerblue"
    Cornsilk = "cornsilk"
    NoColor = "none"


class SecurityClearanceLevel(Enum):
    Unclassified = "unclassified"
    Confidential = "confidential"
    Secret = "secret"
    TopSecret = "top-secret"



import grpc
from github.com.metaprov.modelaapi.pkg.apis.infra.v1alpha1.generated_pb2 import Attachment as MDAttachment
from github.com.metaprov.modelaapi.services.attachment.v1.attachment_pb2_grpc import AttachmentServiceStub
from github.com.metaprov.modelaapi.services.attachment.v1.attachment_pb2 import CreateAttachmentRequest, \
    UpdateAttachmentRequest, \
    DeleteAttachmentRequest, GetAttachmentRequest, ListAttachmentsRequest

from modela.Resource import Resource
from modela.ModelaException import ModelaException
from typing import List, Union


class Attachment(Resource):
    def __init__(self, item: MDAttachment = MDAttachment(), client=None, namespace="", name=""):
        super().__init__(item, client, namespace=namespace, name=name)


class AttachmentClient:
    def __init__(self, stub, modela):
        self.modela = modela
        self.__stub: AttachmentServiceStub = stub

    def create(self, attachment: Attachment) -> bool:
        request = CreateAttachmentRequest()
        request.attachment.CopyFrom(attachment.raw_message)
        try:
            response = self.__stub.CreateAttachment(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def update(self, attachment: Attachment) -> bool:
        request = UpdateAttachmentRequest()
        request.attachment.CopyFrom(attachment.raw_message)
        try:
            self.__stub.UpdateAttachment(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def get(self, namespace: str, name: str) -> Union[Attachment, bool]:
        request = GetAttachmentRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.GetAttachment(request)
            return Attachment(response.attachment, self)
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def delete(self, namespace: str, name: str) -> bool:
        request = DeleteAttachmentRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.DeleteAttachment(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def list(self, namespace: str) -> Union[List[Attachment], bool]:
        request = ListAttachmentsRequest()
        request.namespace = namespace
        try:
            response = self.__stub.ListAttachments(request)
            return [Attachment(item, self) for item in response.attachments.items]
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False





import grpc
from github.com.metaprov.modelaapi.pkg.apis.data.v1alpha1.generated_pb2 import WebRequestRun as MDWebRequestRun
from github.com.metaprov.modelaapi.services.webrequestrun.v1.webrequestrun_pb2_grpc import WebRequestRunServiceStub
from github.com.metaprov.modelaapi.services.webrequestrun.v1.webrequestrun_pb2 import CreateWebRequestRunRequest, \
    UpdateWebRequestRunRequest, \
    DeleteWebRequestRunRequest, GetWebRequestRunRequest, ListWebRequestRunsRequest

from modela.Resource import Resource
from modela.ModelaException import ModelaException
from typing import List, Union


class WebRequestRun(Resource):
    def __init__(self, item: MDWebRequestRun = MDWebRequestRun(), client=None, namespace="", name="", version=Resource.DefaultVersion):
        super().__init__(item, client, namespace=namespace, name=name, version=version)


class WebRequestRunClient:
    def __init__(self, stub, modela):
        self.modela = modela
        self.__stub: WebRequestRunServiceStub = stub

    def create(self, webrequestrun: WebRequestRun) -> bool:
        request = CreateWebRequestRunRequest()
        request.webrequestrun.CopyFrom(webrequestrun.raw_message)
        try:
            response = self.__stub.CreateWebRequestRun(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def update(self, webrequestrun: WebRequestRun) -> bool:
        request = UpdateWebRequestRunRequest()
        request.webrequestrun.CopyFrom(webrequestrun.raw_message)
        try:
            self.__stub.UpdateWebRequestRun(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def get(self, namespace: str, name: str) -> Union[WebRequestRun, bool]:
        request = GetWebRequestRunRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.GetWebRequestRun(request)
            return WebRequestRun(response.webrequestrun, self)
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def delete(self, namespace: str, name: str) -> bool:
        request = DeleteWebRequestRunRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.DeleteWebRequestRun(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def list(self, namespace: str) -> Union[List[WebRequestRun], bool]:
        request = ListWebRequestRunsRequest()
        request.namespace = namespace
        try:
            response = self.__stub.ListWebRequestRuns(request)
            return [WebRequestRun(item, self) for item in response.webrequestruns.items]
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False





import asyncio
import json
import time

import grpc
from google.protobuf import json_format

from github.com.metaprov.modelaapi.pkg.apis.inference.v1alpha1.generated_pb2 import Predictor as MDPredictor
from github.com.metaprov.modelaapi.services.grpcinferenceservice.v1.grpcinferenceservice_pb2 import \
    PredictResultLineItem
from github.com.metaprov.modelaapi.services.predictor.v1.predictor_pb2_grpc import PredictorServiceStub
from github.com.metaprov.modelaapi.services.predictor.v1.predictor_pb2 import CreatePredictorRequest, \
    UpdatePredictorRequest, \
    DeletePredictorRequest, GetPredictorRequest, ListPredictorsRequest, PredictOneRequest

from modela.common import ObjectReference
from modela.Resource import Resource
from modela.ModelaException import ModelaException, ResourceNotFoundException
from typing import List, Union

from modela.inference.InferenceService import InferenceService
from modela.inference.common import AccessType
from modela.inference.models import ModelDeploymentSpec, PredictorSpec, PredictorStatus, PredictionResult, AutoScalingSpec
from modela.infra.ServingSite import ServingSite
from modela.infra.models import Workload
from modela.training.Model import Model


class Predictor(Resource):
    def __init__(self, item: MDPredictor = MDPredictor(), client=None, namespace="", name="",
                 version=Resource.DefaultVersion,
                 serving_site: Union[ObjectReference, ServingSite, str] = "default-serving-site",
                 model: Union[Model, str] = None,
                 models: List[ModelDeploymentSpec] = None,
                 port: int = 3000,
                 path: str = None,
                 access_type: AccessType = None,
                 replicas: int = 0,
                 autoscale: AutoScalingSpec = None,
                 workload: Workload = None):
        """

        :param client: The Predictor client repository, which can be obtained through an instance of Modela.
        :param namespace: The target namespace of the resource.
        :param name: The name of the resource.
        :param version: The version of the resource.
        :param serving_site: The object reference, Serving Site object, or name under the tenant of the resource for which the
            predictor will be deployed under.
        :param model: The Model object or name that the predictor will serve predictions for. This parameter
            will instantiate the predictor with a singe default deployment specification for the model.
        :param models: The list of model deployment specifications that will be applied to the predictor.
        :param port: The port of the predictor, which if applicable will expose the service internally or externally.
        :param path: The path that the predictor will expose a service on. If this parameter is not specified, then
            a path will be generated based on the access type and predictor name.
        :param access_type: The access type of the Predictor. See https://www.modela.ai/docs/docs/serving/production/
            for documentation on how different access types expose the Predictor service
        :param replicas: The amount of replicas for the predictor, which if greater than zero will serve the prediction
            service on multiple pods.
        :param autoscale: If set to true, the predictor's deployment will scale based on the amount of incoming traffic
            to the service.
        :param workload: The workload specification which determines the resources which will be allocated to the
            prediction service.
        """
        self.default_resource = False
        super().__init__(item, client, namespace=namespace, name=name, version=version)

        spec = self.spec
        if type(serving_site) == ServingSite:
            serving_site = serving_site.reference
        elif type(serving_site) == str:
            serving_site = ObjectReference(Namespace=client.modela.tenant, Name=serving_site)

        if model is not None:
            if type(model) == Model:
                model = model.name
            self.spec.Models = [ModelDeploymentSpec(ModelName=model)]
        else:
            self.spec.Models = models

        if self.default_resource:
            self.spec.ServingsiteRef = serving_site
            if port is not None:
                spec.Port = port

            if path is not None:
                spec.Path = path

            if access_type is not None:
                spec.AccessType = access_type.value

        if replicas > 0:
            spec.Replicas = replicas

        if workload is not None:
            spec.Resources = workload

        if autoscale is not None:
            spec.AutoScaling = autoscale

    @property
    def spec(self) -> PredictorSpec:
        return PredictorSpec().copy_from(self._object.spec)

    @property
    def status(self) -> PredictorStatus:
        return PredictorStatus().copy_from(self._object.status)

    def default(self):
        self.default_resource = True
        PredictorSpec().apply_config(self._object.spec)

    @property
    def model(self) -> Model:
        """ Get the model associated with the Predictor """
        self.ensure_client_repository()
        return self._client.modela.Model(self.namespace, self.spec.Models[0].ModelName)

    async def wait_until_ready(self):
        """ Block until the predictor service is ready to accept prediction requests. """
        payload = self.model.test_prediction
        while True:
            try:
                self.predict(payload)
            except ResourceNotFoundException:
                await asyncio.sleep(1 / 5)
                continue

            return

    def predict(self, predictions: Union[str, dict, List[dict]]) -> List[PredictionResult]:
        """
        Send a batch prediction request

        :param predictions: The JSON payload or dictionary(s) that contain each features name and their input value
        """
        self.ensure_client_repository()
        if type(predictions) == dict:
            predictions = json.dumps([predictions])
        elif type(predictions) == list:
            predictions = json.dumps(predictions)

        return self._client.predict(self.namespace, self.name, "", predictions)

    def connect(self, node_ip="", port_forward=False, tls_cert: str = None) -> InferenceService:
        """
        Connect attempts to make a connection to the gRPC inference service client associated with the Predictor.

        Note that if the Predictor access type is ClusterIP, the SDK will attempt to connect to the Predictor service
        using its cluster-internal DNS (i.e. my-predictor.default-serving-site.svc.cluster.local), of which can
        only be accessed by containers running on the cluster.

        :param node_ip: If the Predictor's access type is of type NodePort, a Kubernetes node IP of the cluster
            hosting the current Modela instance must be provided. This IP can be found by executing the command
            `kubectl get nodes -o wide`
        :param port_forward: If enabled, the SDK will attempt to port forward the Predictor service using kubectl. Kubectl
            must be installed and must be connected to the cluster with the Predictor is running on.
        :param tls_cert: If the Predictor's access type is Ingress, the TLS certificate of the Predictor's Serving Site
           must be provided.
        :return: The InferenceService client connected to the prediction proxy service
        """
        assert node_ip != "" or self.spec.AccessType != AccessType.NodePort

        if port_forward:
            return InferenceService(port_forward=True,
                                    service_namespace=self.spec.ServingsiteRef.Name,
                                    service_name=self.name)

        if self.spec.AccessType == AccessType.NodePort or self.spec.AccessType == AccessType.LoadBalancer:
            return InferenceService((node_ip if node_ip != "" else self.spec.Path), self.spec.NodePort)
        elif self.spec.AccessType == AccessType.Ingress:
            return InferenceService(self.spec.Path, tls_cert=tls_cert)
        elif self.spec.AccessType == AccessType.ClusterIP:
            return InferenceService(self.spec.Path, self.spec.Port)


class PredictorClient:
    def __init__(self, stub, modela):
        self.modela = modela
        self.__stub: PredictorServiceStub = stub

    def create(self, predictor: Predictor) -> bool:
        request = CreatePredictorRequest()
        request.predictor.CopyFrom(predictor.raw_message)
        try:
            response = self.__stub.CreatePredictor(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def update(self, predictor: Predictor) -> bool:
        request = UpdatePredictorRequest()
        request.predictor.CopyFrom(predictor.raw_message)
        try:
            self.__stub.UpdatePredictor(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def get(self, namespace: str, name: str) -> Predictor:
        request = GetPredictorRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.GetPredictor(request)
            return Predictor(response.predictor, self)
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)

    def delete(self, namespace: str, name: str) -> bool:
        request = DeletePredictorRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.DeletePredictor(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def list(self, namespace: str, labels: dict = None) -> List[Predictor]:
        request = ListPredictorsRequest()
        request.namespace = namespace
        if labels is not None:
            request.labels.update(labels)

        try:
            response = self.__stub.ListPredictors(request)
            return [Predictor(item, self) for item in response.predictors.items]
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)

    def predict(self, namespace: str, name: str, fields: str, values: str) -> List[PredictionResult]:
        request = PredictOneRequest()
        request.namespace = namespace
        request.name = name
        request.fields = fields
        request.values = values

        try:
            response = self.__stub.PredictOne(request)
            output = json.loads(response.label)
            print(output)
            return [PredictionResult().from_dict(item) for item in output["items"]]
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False



import grpc
from github.com.metaprov.modelaapi.pkg.apis.inference.v1alpha1.generated_pb2 import CronPrediction as MDCronPrediction
from github.com.metaprov.modelaapi.services.cronprediction.v1.cronprediction_pb2_grpc import CronPredictionServiceStub
from github.com.metaprov.modelaapi.services.cronprediction.v1.cronprediction_pb2 import CreateCronPredictionRequest, \
    UpdateCronPredictionRequest, \
    DeleteCronPredictionRequest, GetCronPredictionRequest, ListCronPredictionsRequest

from modela.Resource import Resource
from modela.ModelaException import ModelaException
from typing import List, Union


class CronPrediction(Resource):
    def __init__(self, item: MDCronPrediction = MDCronPrediction(), client=None, namespace="", name="", version=Resource.DefaultVersion):
        super().__init__(item, client, namespace=namespace, name=name, version=version)


class CronPredictionClient:
    def __init__(self, stub, modela):
        self.modela = modela
        self.__stub: CronPredictionServiceStub = stub

    def create(self, cronprediction: CronPrediction) -> bool:
        request = CreateCronPredictionRequest()
        request.cronprediction.CopyFrom(cronprediction.raw_message)
        try:
            response = self.__stub.CreateCronPrediction(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def update(self, cronprediction: CronPrediction) -> bool:
        request = UpdateCronPredictionRequest()
        request.cronprediction.CopyFrom(cronprediction.raw_message)
        try:
            self.__stub.UpdateCronPrediction(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def get(self, namespace: str, name: str) -> Union[CronPrediction, bool]:
        request = GetCronPredictionRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.GetCronPrediction(request)
            return CronPrediction(response.cronprediction, self)
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def delete(self, namespace: str, name: str) -> bool:
        request = DeleteCronPredictionRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.DeleteCronPrediction(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def list(self, namespace: str) -> Union[List[CronPrediction], bool]:
        request = ListCronPredictionsRequest()
        request.namespace = namespace
        try:
            response = self.__stub.ListCronPredictions(request)
            return [CronPrediction(item, self) for item in response.cronpredictions.items]
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False





from dataclasses import field
import github.com.metaprov.modelaapi.pkg.apis.catalog.v1alpha1.generated_pb2 as catalog_pb
import github.com.metaprov.modelaapi.pkg.apis.infra.v1alpha1.generated_pb2 as infra_pb
import github.com.metaprov.modelaapi.pkg.apis.data.v1alpha1.generated_pb2 as data_pb
from modela.infra.common import *
from modela.Configuration import *
from modela.common import ObjectReference


@datamodel(proto=catalog_pb.NotificationSpec)
class NotificationSettings(Configuration):
    """ NotificationSpec specifies which Notifiers to forward Alert resources to """
    ErrorTTL: int = 3600
    """ Time-to-live for error messages, in seconds """
    SuccessTTL: int = 3600
    """ Time-to-live for success messages. In seconds """
    NotifierName: str = ''
    """ The name of the Notifier which exists in the same tenant as the resource specifying the NotificationSpec """


@datamodel(proto=catalog_pb.ResourceSpec)
class Workload(Configuration):
    """ ResourceSpec specifies the amount of resources that will be allocated to a workload """
    WorkloadName: str = ''
    """
    If this resource is based on the workload, this field contain the name of the workload.
    The name of a WorkloadClass. The system will use the resource requirements described by the WorkloadClass
    """
    CpuImage: ObjectReference = ObjectReference('modela-catalog', 'modela-cpu-trainer-latest')
    """ Reference to the managed CPU trainer image, used internally """
    GpuImage: ObjectReference = ObjectReference('modela-catalog', 'modela-gpu-trainer-latest')
    """ Reference to the managed GPU trainer image, used internally """


@datamodel(proto=catalog_pb.ContainerLog)
class ContainerLog(Configuration):
    """
    Append the logs only if they are not already exists
    ContainerLog describes the location of logs for a single Job
    """
    Job: str = ''
    """ The name of the Job """
    Container: str = ''
    """ The container name """
    Key: str = ''
    """ The path to the log in the bucket """


@datamodel(proto=catalog_pb.Logs)
class OutputLogs(Configuration):
    """ Logs describes the location of logs produced by workloads associated with a resource """
    BucketName: str = ''
    """ The name of the VirtualBucket resource where the logs are stored """
    Containers: List[ContainerLog] = field(default_factory=lambda : [])
    """ The collection of ContainerLog objects that describe the location of logs per container """


@datamodel(proto=data_pb.GitLocation)
class GitSettings(Configuration):
    """ GitLocation specifies the Git location where Modela will track resources as YAML """
    GitConnectionName: str = ''
    """ The Git Connection resource which exists in the same tenant as the parent DataProduct """
    Url: str = ''
    """ The URL to the destination Git repository """
    Branch: str = ''
    """ The branch inside the Git repository """
    Private: bool = False
    """ Indicates if the repository is private """


@datamodel(proto=data_pb.ImageLocation)
class ImageLocation(Configuration):
    """ ImageLocation specifies the destination for all model images produced under a DataProduct """
    Name: str = ''
    """ The canonical name of the image repository. If not set, it will default to docker/{dataproduct_name} """
    RegistryConnectionName: str = ''
    """
    The image repository Connection resource which exists in the same tenant as the parent DataProduct. If the field
    is not set, Modela will ignore the image location and not push images
    """


@datamodel(proto=infra_pb.RuleSpec)
class Rule(Configuration):
    """ RuleSpec defines the relation between a resource and the actions that can be performed on the resource """
    Resource: ResourceKind = None
    """ The kind of the resource """
    Verbs: List[Verb] = field(default_factory=lambda : [])
    """ List of allowed actions on the resource """


@datamodel(proto=infra_pb.UserRoleClassSpec)
class UserRoleClassSpec(Configuration):
    """ UserRoleClassSpec contains the permissions for a UserRoleClass """
    TenantRef: ObjectReference = None
    """ The owner of the user role class """
    Description: str = ''
    """ The description of the user role class. """
    Owner: str = ''
    """ The name of the Account which created the object, which exists in the same tenant as the object """
    Rules: List[Rule] = field(default_factory=lambda : [])
    """ The collection of rules, consisting of a resource and the actions that can be performed on the resource """



from dataclasses import dataclass
from typing import List, get_type_hints, get_args
import typing_utils
from google.protobuf.message import Message
from enum import Enum
from modela.util import TrackedList


def isPrimitive(obj):
    return not hasattr(obj, '__dict__')


def convert_case(attr):
    return attr[0].lower() + attr[1:]


class Configuration(object):
    """
    Configuration is a base class for all dataclasses which reflect Protobuf Messages representing Modela API models.
    It serves to provide fully typed (and tracked) interactions with the Modela API. This class
    provides methods for interacting with raw Protobuf Message types and automatically reading and propagating changes
    from a Python dataclass to a Protobuf Message, and vice-versa.
    """

    def __post_init__(self):  # Apply tracking to all list attributes post-initialization
        for attribute, value in self.__dict__.items():
            annotation_type = get_type_hints(self)[attribute]
            if typing_utils.issubtype(annotation_type, List):
                setattr(self, attribute, TrackedList(value, self, attribute))

    def to_message(self):
        if hasattr(self, "proto_type"):
            return self.set_parent(self.proto_type()).parent
        else:
            raise TypeError("Configuration does not have any protobuf information.")

    def apply_field(self, attribute, value, message: Message):
        """ Apply a single attribute to a Protobuf Message."""
        if value is None:
            return
        real_field = convert_case(attribute)
        annotation_type = get_type_hints(self)[attribute]
        if typing_utils.issubtype(annotation_type, List):
            del getattr(message, real_field)[:]
            if issubclass(get_args(annotation_type)[0], Enum): 
                getattr(message, real_field).extend([model.value for model in value])
            elif isPrimitive(get_args(annotation_type)[0]()):
                getattr(message, real_field).extend(value)
            else:
                getattr(message, real_field).extend([model.to_message() for model in value])
                for ind, model in enumerate(
                        value):  # Fix _parent references, protobuf containers like to make copies of objects
                    model._parent = getattr(message, real_field)[ind]

        elif isinstance(getattr(message, real_field), Message):
            value.set_parent(getattr(message, real_field))
        else:
            setattr(message, real_field, value.value if isinstance(value, Enum) else (value or annotation_type()))

    def apply_config(self, message: Message):
        """
        Apply all attributes of a Configuration to a Protobuf Message. If the message does not match the fields of the
        Configuration, an error will be thrown.
        """
        for attribute, value in self.__dict__.items():
            if attribute == "_parent":
                continue
            self.apply_field(attribute, value, message)

        return message

    def copy_from(self, message: Message):
        """
        Copy the attributes of a Protobuf Message to the attributes of the Configuration dataclass.

        :param message: The Protobuf Message to copy data from.
        :rtype: Configuration
        """
        for attribute, value in self.__dict__.items():
            if attribute == "_parent":
                continue
            annotation_type = get_type_hints(self)[attribute]
            message_attr = getattr(message, convert_case(attribute))
            if typing_utils.issubtype(annotation_type, List):
                annotation_type = get_args(annotation_type)[0]
                if issubclass(annotation_type, Enum) or isPrimitive(annotation_type()):
                    setattr(self, attribute, TrackedList([annotation_type(message) for message in message_attr],
                                                         self, attribute))
                else:
                    setattr(self, attribute, TrackedList(
                        [annotation_type().copy_from(message).set_parent(message) for message in message_attr],
                        self, attribute))
            elif isinstance(message_attr, Message):
                if message_attr.ByteSize() <= 0:
                    continue
                setattr(self, attribute, annotation_type().copy_from(message_attr))
            else:
                try:
                    setattr(self, attribute,
                            (type(annotation_type) if not callable(annotation_type) else annotation_type)(
                                message_attr))
                except ValueError:
                    pass

        self._parent = message
        return self

    def from_dict(self, data: dict):
        for attribute, value in self.__dict__.items():
            if attribute == "_parent" or not convert_case(attribute) in data:
                continue

            annotation_type = get_type_hints(self)[attribute]
            dict_attr = data[convert_case(attribute)]
            if typing_utils.issubtype(annotation_type, List):
                annotation_type = get_args(annotation_type)[0]
                if issubclass(annotation_type, Enum) or isPrimitive(annotation_type()):
                    setattr(self, attribute, TrackedList([annotation_type(message) for message in dict_attr],
                                                         self, attribute))
                else:
                    setattr(self, attribute, TrackedList(
                        [annotation_type().from_dict(message) for message in dict_attr],
                        self, attribute))
            elif isinstance(dict_attr, dict):
                setattr(self, attribute, annotation_type().from_dict(data))
            else:
                try:
                    setattr(self, attribute,
                            (type(annotation_type) if not callable(annotation_type) else annotation_type)(
                                dict_attr))
                except ValueError:
                    pass

        return self

    def set_parent(self, message: Message):
        """
        Set the Configuration's related Protobuf Message. Each attribute of the configuration will be applied to the message.
        If `message` does not contain the exact attributes of the configuration the function will fail. All future changes
        to the Configuration will be propagated to the message.

        :rtype: Configuration
        :param message: Protobuf Message which represents this Configuration.
        """
        self._parent = message
        self.apply_config(message)
        return self

    def propagate_to_parent(self, attribute, value):
        if hasattr(self, "_parent"):
            self.apply_field(attribute, value, self._parent)

    @property
    def parent(self):
        return self._parent

    def __setattr__(self, attribute, value):
        if hasattr(self, "_parent") and attribute != "_parent":
            self.apply_field(attribute, value, self._parent)

        if type(value) == list:
            value = TrackedList(value, self, attribute)

        super().__setattr__(attribute, value)

class ImmutableConfiguration(Configuration):
    """
    ImmutableConfiguration represents Modela API objects which should not be changed by end-users.
    """

    def __setattr__(self, attribute, value):
        object.__setattr__(self, attribute, value)

def datamodel(proto):
    """
    Decorator for Configurations to wrap them in a dataclass and store additional information about their related Protobuf message
    """
    def wrap(cls):
        setattr(cls, "proto_type", proto)
        return dataclass(cls)

    return wrap


import grpc
from github.com.metaprov.modelaapi.pkg.apis.training.v1alpha1.generated_pb2 import NotebookRun as MDNotebookRun
from github.com.metaprov.modelaapi.services.notebookrun.v1.notebookrun_pb2_grpc import NotebookRunServiceStub
from github.com.metaprov.modelaapi.services.notebookrun.v1.notebookrun_pb2 import CreateNotebookRunRequest, \
    UpdateNotebookRunRequest, \
    DeleteNotebookRunRequest, GetNotebookRunRequest, ListNotebookRunsRequest

from modela.Resource import Resource
from modela.ModelaException import ModelaException
from typing import List, Union


class NotebookRun(Resource):
    def __init__(self, item: MDNotebookRun = MDNotebookRun(), client=None, namespace="", name="", version=Resource.DefaultVersion):
        super().__init__(item, client, namespace=namespace, name=name, version=version)


class NotebookRunClient:
    def __init__(self, stub, modela):
        self.modela = modela
        self.__stub: NotebookRunServiceStub = stub

    def create(self, notebookrun: NotebookRun) -> bool:
        request = CreateNotebookRunRequest()
        request.notebookrun.CopyFrom(notebookrun.raw_message)
        try:
            response = self.__stub.CreateNotebookRun(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def update(self, notebookrun: NotebookRun) -> bool:
        request = UpdateNotebookRunRequest()
        request.notebookrun.CopyFrom(notebookrun.raw_message)
        try:
            self.__stub.UpdateNotebookRun(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def get(self, namespace: str, name: str) -> Union[NotebookRun, bool]:
        request = GetNotebookRunRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.GetNotebookRun(request)
            return NotebookRun(response.notebookrun, self)
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def delete(self, namespace: str, name: str) -> bool:
        request = DeleteNotebookRunRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.DeleteNotebookRun(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def list(self, namespace: str) -> Union[List[NotebookRun], bool]:
        request = ListNotebookRunsRequest()
        request.namespace = namespace
        try:
            response = self.__stub.ListNotebookRuns(request)
            return [NotebookRun(item, self) for item in response.notebookruns.items]
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False


